%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  プログラム 10.1:速度観測での
% 　外乱推定カルマンフィルタ(KFD)の例
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;clc;
%% 物理パラメータ
m0=1;d=10;            % 質量[kg]と外乱の大きさ[N]
T=0.01;               % 制御周期[s]
zeta=7.5;wn=5;        % PI制御系設計のための減衰係数と応答角周波数
Kp=2*zeta*wn;Ki=wn^2; % PI制御ゲイン

%% 状態空間モデル
A=[0,];B=[1/m0];C=[1];D=0;sysc=ss(A,B,C,D);
sysd=c2d(sysc,T);[Ad,Bd,Cd,Dd]=ssdata(sysd);X_0=[0];
    
%% 拡大系の状態空間モデル
OA=[Ad,-Bd;0,1];OB=[Bd;0];OC=[Cd,0];OD=0;OBv=eye(2);

%% ノイズの設計
Qkl=diag([2e3,1e6]); % 仮想的なシステムノイズの共分散行列の例
Rkl=0.001;           % 観測ノイズの共分散の例
dt=0.005;            % 観測ノイズの周期

%% カルマンフィルタ設計のための初期パラメータの設定
gamma=1;P_0=gamma*eye(2); % gammaは共分散行列Pの初期値の大きさ
x_hat_0=[0;0];            % 状態推定値の初期値

%% シミュレーション
tend=5; % 終了時間 
open_system('sim_Figure_10_2_and_10_3_PI_control_with_KFD');
set_param('sim_Figure_10_2_and_10_3_PI_control_with_KFD','WideLines','on'); 
set_param('sim_Figure_10_2_and_10_3_PI_control_with_KFD','ShowLineDimensions','on');
y=sim('sim_Figure_10_2_and_10_3_PI_control_with_KFD');

%% 描画
% Scope'd&d_hat'のログの変数名をd_outに設定
% Scope'Vc_ref&Vc'のログの変数名をv_outに設定
% Scopeのログの保存形式を「時間付き構造体」とする
t=y.d_out.time;d=y.d_out.signals(1).values;
d=y.d_out.signals(1).values;
d_hat=y.d_out.signals(2).values;
figure(1);clf(1);plot(t,d,'k--',t,d_hat,'k-','LineWidth',1.2);grid on;
xlabel('Time[s]','Fontsize',16);ylabel('Disturbance[N]','Fontsize',12);
axis([0,tend,-10,20]);
legend('Disturbance', 'Disturbance estimate','Location','Southeast','Fontsize',12);
title('Figure 10.3 (a)');
set(gca,'Fontname','Times New Roman','FontSize',14);  % 文字の設定
set(gcf,'color','w');   % 枠の周囲を白く変更する
% %
vc_ref=y.v_out.signals(1).values;vc=y.v_out.signals(2).values;
vc_hat=y.v_out.signals(3).values;
figure(2);clf(2);plot(t,vc_ref,'k--',t,vc,'k-.',t,vc_hat,'k-','LineWidth',1.2);grid on;
xlabel('Time[s]','Fontsize',16);ylabel('Velocity[m/s]','Fontsize',12);
axis([0,tend,-0.2,1.2]);
legend('Velocity-ref', 'Velocity','Vel.estimate','Location','Southeast','Fontsize',12);
title('Figure 10.3 (b)');
set(gca,'Fontname','Times New Roman','FontSize',14);  % 文字の設定
set(gcf,'color','w');   % 枠の周囲を白く変更する

delete('*.slxc');delete('*.asv');