%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  プログラム 10.3:位置観測での
% 　外乱推定カルマンフィルタ併用サーボ系
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;clc;
%% 物理パラメータ
m=1;d=10;       % 質量[kg]と外乱の大きさ[N]
T=0.01;         % 制御周期[s]

%% 状態空間モデル
A=[0,1;0,0];B=[0;1/m];C=[1,0];D=0;
[Ad,Bd,Cd,Dd]=c2dm(A,B,C,D,T);x0=[0;0];
    
%% KFD設計のための拡大系の状態空間モデル
OA=[Ad,-Bd;zeros(1,2),1];OB=[Bd;0];OC=[Cd,0];OD=0;OBv=eye(3);

%% ノイズの設計
Qx=2e-4;Qv=6e-4;Qd=2e-1;Q=diag([Qx,Qv,Qd]); RR=2e-3;% ノイズの共分散行列
Qklx=Qx;Qklv=Qv;Qkld=Qd;Qkl=diag([Qklx,Qklv,Qkld]); % KFDでのシステムノイズの共分散行列
Rkl=RR;   % KFDでの観測ノイズの共分散行列
dt=0.005; % 観測ノイズのサンプル時間

%% カルマンフィルタ設計のための初期パラメータの設定
gamma=1;P_0=gamma*eye(3); % gammaは共分散行列Pの初期値の大きさ
x_hat_0=[0;0;0];          % 位置,速度,外乱推定値の初期値

%% サーボ系設計
Adbar=[Ad,zeros(2,1);-Cd*T,1];Bdbar=[Bd;0];
Q=diag([8e1,3e2,2e7]);R=1;Fd_bar=dlqr(Adbar,Bdbar,Q,R);
Fd=Fd_bar(1:2);Ki=-Fd_bar(3);

%% シミュレーション
open('sim_Figure_10_4_and_10_5_pos_servo_with_KFD');
tend=5;Xc_ref=1; % 終了時間,位置参照値
set_param('sim_Figure_10_4_and_10_5_pos_servo_with_KFD','WideLines','on'); 
set_param('sim_Figure_10_4_and_10_5_pos_servo_with_KFD','ShowLineDimensions','on');
y=sim('sim_Figure_10_4_and_10_5_pos_servo_with_KFD');

%% 描画
% Scope'd&d_hat'のログの変数名をd_outに設定する
% Scope'XC_ref&Xc 1'のログの変数名をx_outに設定する
% Scopeのログの保存形式を「時間付き構造体」とする
t=y.d_out.time;d=y.d_out.signals(1).values;
d=y.d_out.signals(1).values;
d_hat=y.d_out.signals(2).values;
figure(1);clf(1);plot(t,d,'k--',t,d_hat,'k-','LineWidth',1.2);grid on;
xlabel('Time[s]','Fontsize',16);ylabel('Disturbance[N]','Fontsize',12);
axis([0,tend,-10,20]);
legend('Disturbance', 'Disturbance estimate','Location','Southeast','Fontsize',12);
title('Figure 10.5 (a-1)');
set(gca,'Fontname','Times New Roman','FontSize',14); % 文字の設定
set(gcf,'color','w');   % 枠の周囲を白く変更する
%
xc_ref=y.x_out.signals(1).values;xc=y.x_out.signals(2).values;
xc_hat=y.x_out.signals(3).values;
figure(2);clf(2);plot(t,xc_ref,'k--',t,xc,'k-.',t,xc_hat,'k-','LineWidth',1.2);grid on;
xlabel('Time[s]','Fontsize',16);ylabel('Position[m]','Fontsize',12);
axis([0,tend,-0.2,1.3]);
legend('Xc-ref', 'Xc','Xc estimate','Location','Southeast','Fontsize',12);
title('Figure 10.5 (a-2)');
set(gca,'Fontname','Times New Roman','FontSize',14); % 文字の設定
set(gcf,'color','w');   % 枠の周囲を白く変更する

delete('*.slxc');delete('*.asv');