%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  プログラム 10.4: 最小次元ディジタルDOB併用 
%         サーボ系　[位置観測型]
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;clc;
%% 物理パラメータ
m=1;d=10;       % 質量[kg]と外乱の大きさ[N]
T=0.01;         % 制御周期[s]

%% 状態空間モデル
A=[0,1;0,0];B=[0;1/m];C=[1,0];D=0;
[Ad,Bd,Cd,Dd]=c2dm(A,B,C,D,T);x0=[0;0];
    
%% KFD設計のための拡大系の状態空間モデル
OA=[Ad,-Bd;zeros(1,2),1];OB=[Bd;0];OC=[Cd,0];OD=0;OBv=eye(3);

%% ノイズの設計
Qx=2e-4;Qv=6e-4;Qd=2e-1;Q=diag([Qx,Qv,Qd]); RR=2e-3;% ノイズの共分散行列
dt=0.005; % 観測ノイズのサンプル時間

%% 最小次元ディジタルDOB設計
% 拡大系の作成と各行列の導出
A_=[A,-B;zeros(1,2),0];B_=[B;0];C_=[C,0];
[Ad_,Bd_,Cd_,Dd_]=c2dm(A_,B_,C_,D,T);
A11=Ad_(1,1);A12=Ad_(1,2:3);A21=Ad_(2:3,1);A22=Ad_(2:3,2:3);
B1=Bd_(1);B2=Bd_(2:3);
% DOB設計
pole=[0.92,0.95];Ltemp=place(A22',A12',pole);L=Ltemp';
Ad_hat=A22-L*A12,Bd_hat=Ad_hat*L+A21-L*A11
Jd_hat=-L*B1+B2,Cd_hat=[zeros(1,2);eye(2)],Dd_hat=[1;L];
x_hat_0=[0;0;0];     % 初期推定値

%% サーボ系設計
Adbar=[Ad,zeros(2,1);-Cd*T,1];Bdbar=[Bd;0];
Q=diag([8e1,3e2,2e7]);R=1;Fd_bar=dlqr(Adbar,Bdbar,Q,R);
Fd=Fd_bar(1:2);Ki=-Fd_bar(3);

%% シミュレーション
tend=5;Xc_ref=1; % 終了時間と位置参照値
open_system('sim_Figure_10_6_and_10_5b_digital_servo_with_min_DOB');
set_param('sim_Figure_10_6_and_10_5b_digital_servo_with_min_DOB','WideLines','on');
set_param('sim_Figure_10_6_and_10_5b_digital_servo_with_min_DOB','ShowLineDimensions','on');
y=sim('sim_Figure_10_6_and_10_5b_digital_servo_with_min_DOB');

%% 描画
% Scope'd&d_hat'のログの変数名をd_outに設定する
% Scope'XC_ref&Xc 1'のログの変数名をx_outに設定する
% Scopeのログの保存形式を「時間付き構造体」とする
t=y.d_out.time;d=y.d_out.signals(1).values;
d=y.d_out.signals(1).values;
d_hat=y.d_out.signals(2).values;
figure(1);clf(1);plot(t,d,'k--',t,d_hat,'k-','LineWidth',1.2);grid on;
xlabel('Time[s]','Fontsize',16);ylabel('Disturbance[N]','Fontsize',12);
axis([0,tend,-10,20]);
legend('Disturbance', 'Est.','Location','Southeast','Fontsize',12);
title('Figure 10.5 (b-1)');
set(gca,'Fontname','Times New Roman','FontSize',14);  % 文字の設定
set(gcf,'color','w');   % 枠の周囲を白く変更する
%
xc_ref=y.x_out.signals(1).values;xc=y.x_out.signals(2).values;
xc_hat=y.x_out.signals(3).values;
figure(2);clf(2);plot(t,xc_ref,'k--',t,xc,'k-.',t,xc_hat,'k-','LineWidth',1.2);grid on;
xlabel('Time[s]','Fontsize',16);ylabel('Position[m]','Fontsize',12);
axis([0,tend,-0.2,1.3]);
legend('Xc-ref', 'Xc','Xc.Estimate','Location','Southeast','Fontsize',12);
title('Figure 10.5 (b-2)');
set(gca,'Fontname','Times New Roman','FontSize',14);  % 文字の設定
set(gcf,'color','w');   % 枠の周囲を白く変更する

delete('*.slxc');delete('*.asv');