%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% プログラム 10.5: 速度計算値を用いた
%       外乱オブザーバ併用位置サーボ系の例
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;clc;
%% 物理パラメータ
m0=1;d=10;                  % 質量公称値[kg]と外乱の大きさ[N]
T=0.01; a=1/T;              % 制御周期[s](=DOBの時定数)
alpha_d=exp(-a*T);          % ディジタルDOBの極
a2=a*5;alpha_d2=exp(-a2*T); % 速度計算用の極

%% 状態空間モデル
A=[0,1;0,0];B=[0;1/m0];C=[1,0];D=0;sysc=ss(A,B,C,D);
sysd=c2d(sysc,T);[Ad,Bd,Cd,Dd]=ssdata(sysd);
    
%% ノイズの設計
Rkl=0.001/2;             % 観測ノイズの共分散
dt=0.005;                % 観測ノイズのサンプル時間

%% サーボ系設計
Adbar=[Ad,zeros(2,1);-Cd*T,1];Bdbar=[Bd;0];
Q=diag([8e2,3e-2,2e4]);R=1;Fd_bar=dlqr(Adbar,Bdbar,Q,R);
Fd=Fd_bar(1:2);Ki=-Fd_bar(3);

%% シミュレーション
tend=5;Xc_ref=1; % 終了時間と位置参照値
open_system('sim_Figure_10_7_digital_pos_con_with_vel_based_DOB');
y=sim('sim_Figure_10_7_digital_pos_con_with_vel_based_DOB');

%% 描画
% Scope'f,d and d_hat'のログの変数名をd_outに設定する
% Scope'XC_ref&Xc'のログの変数名をx_outに設定する
% Scopeのログの保存形式を「時間付き構造体」とする
t=y.d_out.time;d=y.d_out.signals(1).values;
f=y.d_out.signals(1).values;d=y.d_out.signals(2).values(:,1);
d_hat=y.d_out.signals(2).values(:,2);
%
figure(1);clf(1);plot(t,d,'k--',t,d_hat,'k-','LineWidth',1.2);grid on;
xlabel('Time[s]','Fontsize',16);ylabel('Disturbance[N]','Fontsize',12);
axis([0,tend,-10,20]);
legend('Disturbance', 'Disturbance Estimate','Location','Southeast','Fontsize',12);
title('Figure 10.8 (a-1) or (b-1)');
set(gca,'Fontname','Times New Roman','FontSize',14); % 文字の設定
set(gcf,'color','w');   % 枠の周囲を白く変更する
%
xc_ref=y.x_out.signals(1).values;xc=y.x_out.signals(2).values;
figure(2);clf(2);plot(t,xc_ref,'k--',t,xc,'k-','LineWidth',1.2);grid on;
xlabel('Time[s]','Fontsize',16);ylabel('Position[m]','Fontsize',12);
axis([0,tend,-0.2,1.2]);
legend('Xc-ref', 'Xc','Location','Southeast','Fontsize',12);
title('Figure 10.8 (a-2) or (b-2)');
set(gca,'Fontname','Times New Roman','FontSize',14); % 文字の設定
set(gcf,'color','w');   % 枠の周囲を白く変更する
%
figure(3);clf(3);plot(t,f,'k','LineWidth',1.2);grid on;
xlabel('Time[s]','Fontsize',16);ylabel('Driving Forcr[N]','Fontsize',12);
axis([0,tend,-10,20]);
title('Figure 10.8 (a-3) or (b-3)');
set(gca,'Fontname','Times New Roman','FontSize',14); % 文字の設定
set(gcf,'color','w');   % 枠の周囲を白く変更する

delete('*.slxc');delete('*.asv');