%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% プログラム 10.6: 外乱推定定常カルマンフィルタ
% 　　　　　　を用いた位置サーボ系の例
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;clc;
%% 物理パラメータ
m0=1;d=10;            % 質量公称値[kg]と外乱の大きさ[N]
T=0.01;               % 制御周期[s]]

%% 状態空間モデル
A=[0,1;0,0];B=[0;1/m0];C=[1,0];D=0;
[Ad,Bd,Cd,Dd]=c2dm(A,B,C,D,T);X_0=[0;0];
    
%% KFD設計のための拡大系の作成
OA=[Ad,-Bd;zeros(1,2),1];OB=[Bd;0];OC=[Cd,0];OD=0;OBv=eye(3);

%% ノイズ設計
Qklx=2e2;Qklv=6e-3;Qkld=7e4;Qkl=diag([Qklx,Qklv,Qkld]); 
% KFD設計のための仮想的なシステムノイズの共分散行列
Rkl=0.001/2;      % 観測ノイズの共分散
dt=0.005;         % 観測ノイズのサンプル周期

%% stationary Kalman Filter Design
set(gca,'Fontname','Times New Roman','FontSize',14); % 文字の設定
set(gcf,'color','w');   % 枠の周囲を白く変更する
%
[H_temp,P,e]=dlqr(OA',OC',Qkl,Rkl);G_k=H_temp';
ee=P-OA*P*OA'+OA*P*OC'*inv(Rkl+OC*P*OC')*OC*P*OA'-Qkl; 
x_hat_0=[0;0;0]; % 初期推定値

%% サーボ系設計
Adbar=[Ad,zeros(2,1);-Cd*T,1];Bdbar=[Bd;0];
Q=diag([8e2,3e-2,2e4]);
R=1;Fd_bar=dlqr(Adbar,Bdbar,Q,R);
Fd=Fd_bar(1:2);Ki=-Fd_bar(3);

%% シミュレーション
tend=5;Xc_ref=1; % 終了時間と位置参照値
open_system('sim_Figure_10_9_and_10_10_pos_con_with_Stationary_KF');
set_param('sim_Figure_10_9_and_10_10_pos_con_with_Stationary_KF','WideLines','on');
set_param('sim_Figure_10_9_and_10_10_pos_con_with_Stationary_KF','ShowLineDimensions','on');
y=sim('sim_Figure_10_9_and_10_10_pos_con_with_Stationary_KF');

%% 描画
% Scope'd&d_hat'のログの変数名をd_outに設定する
% Scope'Xc_ref&Xc 1'のログの変数名をx_outに設定する
% Scopeのログの保存形式を「時間付き構造体」とする
t=y.d_out.time;d=y.d_out.signals(1).values;
d=y.d_out.signals(1).values;
d_hat=y.d_out.signals(2).values;
figure(1);clf(1);plot(t,d,'k--',t,d_hat,'k-','LineWidth',1.2);grid on;
xlabel('Time[s]','Fontsize',16);ylabel('Disturbance[N]','Fontsize',12);
axis([0,tend,-10,20]);
legend('Disturbance', 'Est.','Location','Southeast','Fontsize',12);
set(gca,'Fontname','Times New Roman','FontSize',14); % 文字の設定
set(gcf,'color','w');   % 枠の周囲を白く変更する
%
xc_ref=y.x_out.signals(1).values;xc=y.x_out.signals(2).values;
xc_hat=y.x_out.signals(3).values;
figure(2);clf(2);plot(t,xc_ref,'k--',t,xc,'k-.',t,xc_hat,'k-','LineWidth',1.2);grid on;
xlabel('Time[s]','Fontsize',16);ylabel('Position[m]','Fontsize',12);
axis([0,tend,-0.2,1.2]);
legend('Xc-ref', 'Xc','Xc.Estimate','Location','Southeast','Fontsize',12);
set(gca,'Fontname','Times New Roman','FontSize',14); % 文字の設定
set(gcf,'color','w');   % 枠の周囲を白く変更する
delete('*.slxc');delete('*asv');