%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% プログラム 10.8:1リンクマニピュレータのためのEKFD併用制御
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;clc;
%------------------------------------------------------
% Xbar(k)=[angle;angular velocity;disturbance];
% f(x)=[theta(k)+T*omega(k); 　　　拡大状態変数
%      omega(k)-Tg/r*cos(theta(k))-1/(mr^2)*dis(k);
%        dis(k)        ];  　　　　式(10.27)
% Ad=[1                ,T,  0;
%    Tg/r*sin(theta_k) ,1, -T/(mr^2);
%     0                ,0,  1 ];　式(10.29)
% Bd=[0; T/(m*r^2);0];Cd=[1 0 0]; 式(10.29)
%-------------------------------------------------------

%% 物理パラメータ 
m=10;r=2.5;g=9.8;c=0.0;        % 質量,半径,重力加速度,粘性摩擦係数
dis=m*r*g*1.0;                 % 外乱の大きさ
dis_time=2.0;                  % 外乱の発生時間
T=0.01;tend=50;                % 制御周期と終了時間
sigma_v_2=1e-3;sigma_w_2=1e-3; % 実際のシステムノイズと観測ノイズ
Qkf=diag([1e-1,1e1,1e7]);
Rkf=1e-6;% EKFDで仮定するシステムノイズと観測ノイズ
theta_0=pi/3;omega_0=0; x_0=[theta_0;omega_0];% 初期値

%% EKFD設計の準備 
Bd=[0;T/(m*r^2)];Cd=[1 0];
Bdbar=[Bd;0];Cdbar=[Cd 0];Bv=eye(3);
theta_hat_0=0;omega_hat_0=0;dis_hat_0=0;
x_hat_0=[theta_hat_0;omega_hat_0;dis_hat_0]; 
P_0=1*eye(3);% 誤差共分散行列の初期値

%% 制御系設計
Q=diag([3e4,2e3]);R=1;
Adf=[1                ,T;
    T*g/r*sin(theta_0),1]; % 角度の初期値を用いて線形化
Fd=dlqr(Adf,Bd,Q,R);

%% シミュレーション
open_system('sim_Figure_10_12_and_10_13_EKFD_1link_manip');
set_param('sim_Figure_10_12_and_10_13_EKFD_1link_manip','WideLines','on');
set_param('sim_Figure_10_12_and_10_13_EKFD_1link_manip',...
    'ShowLineDimensions','on');
z=sim('sim_Figure_10_12_and_10_13_EKFD_1link_manip');

%% 描画
% Scope'real_y(k)&y_hat(k)'のログの変数名をy_outに設定する
% Scope'dis_hat(k)'のログの変数名をd_outに設定する
% ログの保存形式を「時間付き構造体」に設定する
figure(1);clf(1);t=z.y_out.time;
data_x=z.y_out.signals(1).values;
est_x=z.y_out.signals(2).values;
plot(t,data_x,'-k',t,est_x,'-.k','LineWidth',1.2);grid on;
xlabel('Time [s]','Fontsize',16);   
ylabel('Angle[rad]','Fontsize',16);
axis([0,tend,-0.8,1.2]);
legend('Real angle','Angle estimate'); 
set(gca,'Fontname','Times New Roman','FontSize',14); % 文字の設定
set(gcf,'color','w');   % 枠の周囲を白く変更する
%
figure(2);clf(2);t_d=z.d_out.time;
data_d=z.d_out.signals(1).values;est_d=z.d_out.signals(2).values;
plot(t_d,data_d,'--k',t_d,est_d,'k','LineWidth',1.2);grid on;
xlabel('Time [s]','Fontsize',16); 
ylabel('Disturbance[Nm]','Fontsize',16);
axis([0,tend,-50,300]);
legend('Disturbance','Disturbance estimsate','Location','southeast');
set(gca,'Fontname','Times New Roman','FontSize',14); % 文字の設定
set(gcf,'color','w');   % 枠の周囲を白く変更する

delete('*.slxc');delete('.asv');