%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% プログラム 11.1: 非最小実現モデルの評価例  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;clc;format short e;
%% 物理パラメータ
m=2.0;c=0.30;k=0.050;

%% 拡大系の状態空間モデルの導出
A=[0, 1; -k/m ,-c/m];B=[0;1/m];C=[1,0];D=0;[n,~]=size(B);
A_bar=[A,-B;zeros(1,3)];B_bar=[B;0];C_bar=[C,0];n_bar=n+1;
%
lambda1=eig(A_bar);x=poly(lambda1);          % 固有値を求めて多項式係数を得る
a3=x(2);a2=x(3);a1=x(4);                     % s^3+a3*s^2+a2*s+a1と定義する
W=[a2,a3,1;a3,1,0;1,0,0];                    % 可観測正準系を作成するためのW行列      
Uo=[C_bar;C_bar*A_bar;C_bar*A_bar^2];S=W*Uo; % 変換行列Sの作成 
A_t=S*A_bar*inv(S);B_t=S*B_bar;C_t=C_bar*inv(S);
% 順番を入れ替えて適応オブザーバ用の可観測正準系を作成
S2=[0,0,1;0,1,0;1,0,0];
A_tilde=S2*A_t*inv(S2);B_tilde=S2*B_t;C_tilde=C_t*inv(S2);

%% 状態変数フィルタの設計
g=[1;zeros(n_bar-2,1)];
L=[zeros(n_bar-2,1),eye(n_bar-2);0,zeros(1,n_bar-2)];
pole_of_k=[-6,-9,-12];                  % 安定なk(s)の固有値の指定
k_temp=poly(pole_of_k);k=k_temp(2:end); % kベクトルの作成
K=[-k',[g';L]];                         % 状態変数フィルタのK行列の作成

%%  alphaとbetaの正確な値を試算
[num,den]=ss2tf(A_bar,B_bar,C_bar,0);
alpha=(den(2:end))',l=-alpha+k';beta=(num(2:end))',
 
%% 制御系設計
Q=diag([2.5e2,1e2]);R=1;Fb=lqr(A,B,Q,R);
 
%% シミュレーション
x0=[1;0];
tend=100;dis_0=0;dis=15;t_dis=20;
open_system('sim_Figure_11_3_and_11_4_nonminimal_realization_model');
set_param('sim_Figure_11_3_and_11_4_nonminimal_realization_model','WideLines','on'); %
set_param('sim_Figure_11_3_and_11_4_nonminimal_realization_model','ShowLineDimensions','on')
z=sim('sim_Figure_11_3_and_11_4_nonminimal_realization_model');
 
%% 描画
% Scope'y(t)&y_Sfilter'のログの変数名をy_outに設定する
% 保存形式を「時間付き構造体」に設定する
t=z.y_out.time;y=z.y_out.signals(1).values;y_filter=z.y_out.signals(2).values;
figure(1);clf(1);
subplot(2,1,1);plot(t,y,'k','LineWidth',1.2);grid on;
axis([0,tend,-2,1.5]);xlabel('Time[s]');ylabel('Real Position[m]'); 
set(gca,'Fontname','Times New Roman','FontSize',14); 
title('Figure 11.4');
subplot(2,1,2);plot(t,y_filter,'k','LineWidth',1.2);grid on;
axis([0,tend,-2,1.5]);xlabel('Time[s]');ylabel('Filtered Position[m]'); 
set(gca,'Fontname','Times New Roman','FontSize',14); % 文字の設定
set(gcf,'color','w');    % 枠の周囲を白く変更する
%%
delete('*.slxc');delete('*.asv');delete('*.autosave');
 
