%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% プログラム 11.2: 適応外乱オブザーバ併用制御系の例 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;clc;format short e;
%% 物理パラメータ
m=2.0;c=0.30;k=0.050;

%% 拡大系の状態空間モデルの導出
A=[0, 1; -k/m ,-c/m];B=[0;1/m];C=[1,0];D=0;[n,~]=size(B);
A_bar=[A,-B;zeros(1,3)];B_bar=[B;0];C_bar=[C,0];n_bar=n+1;
%
lambda1=eig(A_bar);x=poly(lambda1);          % 固有値を求めて多項式係数を得る
a3=x(2);a2=x(3);a1=x(4);                     % s^3+a3*s^2+a2*s+a1と定義する
W=[a2,a3,1;a3,1,0;1,0,0];                    % 可観測正準系を作成するためのW行列 
Uo=[C_bar;C_bar*A_bar;C_bar*A_bar^2];S=W*Uo; % 変換行列Sの作成
A_t=S*A_bar*inv(S);B_t=S*B_bar;C_t=C_bar*inv(S);
% 順番を入れ替えて適応オブザーバ用の可観測正準系を作成
S2=[0,0,1;0,1,0;1,0,0];
A_tilde=S2*A_t*inv(S2);B_tilde=S2*B_t;C_tilde=C_t*inv(S2);

%% 状態変数フィルタの設計
g=[1;zeros(n_bar-2,1)];
L=[zeros(n_bar-2,1),eye(n_bar-2);0,zeros(1,n_bar-2)];
pole_k=[-3,-4,-2];kk_temp=poly(pole_k);kk=(kk_temp(2:4))';
K=[-kk,[g';L]] % 状態変数フィルタのK行列の作成

%% Kreisselmeierフィルタの設計
C_tilde_K_matrix=[C_tilde;C_tilde*K;C_tilde*K^2];
inv_C_tilde_K_matrix=inv(C_tilde_K_matrix);

%%  評価用：正確なalphaとbetaの値を計算
[num,den]=ss2tf(A_bar,B_bar,C_bar,0);
alpha=(den(2:end))';l=-alpha+kk;beta=(num(2:end))';
alpha_beta=[alpha;beta];l_beta=[l;beta];

%% 適応則ゲインの設定
lambda=1.5e-1;gamma0=5*eye(6);
  
%% MATLAB functionで実行される極配置法での極と設計変数の準備
pole=[-2,-3];h=poly(pole);h1=h(3);h2=h(2);

%% シミュレーション
x0=[0;0];
tend=200;dis_0=5.0;dis=5;t_dis=20;T_ratio=50;
open_system('sim_Figure_11_5_6_7_8_adaptive_DOB_and_Regulator');
set_param('sim_Figure_11_5_6_7_8_adaptive_DOB_and_Regulator','WideLines','on'); 
set_param('sim_Figure_11_5_6_7_8_adaptive_DOB_and_Regulator','ShowLineDimensions','on')
z=sim('sim_Figure_11_5_6_7_8_adaptive_DOB_and_Regulator');
 
%% 描画
% Scope'd&d_hat'のログの変数名をd_outに設定する
% Scope'x(t)'のログの変数名をx_outに設定する
%　Scopeのログの保存形式を「時間付き構造体」に設定する
t=z.d_out.time;
d=z.d_out.signals(1).values;
d_hat=z.d_out.signals(2).values;
figure(1);clf(1);
plot(t,d,'k--',t,d_hat,'k','LineWidth',1.2);grid on;
axis([0,tend,-5,10]);xlabel('Time[s]');ylabel('Disturbance[N]'); 
legend('Disturbance','Disturbance estimate','Location','Southeast');
title('Figure 11.7');
set(gca,'Fontname','Times New Roman','FontSize',14); % 文字の設定
set(gcf,'color','w');   % 枠の周囲を白く変更するr
%
t=z.x_out.time;
xc=z.x_out.signals(1).values(:,1);vc=z.x_out.signals(1).values(:,2);
xc_hat=z.x_out.signals(2).values(:,1);vc_hat=z.x_out.signals(2).values(:,2);
%
figure(2);clf(2);
plot(t,xc,'k--',t,xc_hat,'k','LineWidth',1.2);grid on;
axis([0,tend,-1.5,1.5]);xlabel('Time[s]');ylabel('Position[m]'); 
legend(' Position',' Position estimate','Location','Northeast');
title('Figure 11.8 (a)');
set(gca,'Fontname','Times New Roman','FontSize',14); % 文字の設定
set(gcf,'color','w');   % 枠の周囲を白く変更する
%
figure(3);clf(3);
plot(t,vc,'k--',t,vc_hat,'k','LineWidth',1.2);grid on;
axis([0,tend,-1.5,1.5]);xlabel('Time[s]');ylabel('Velocity[m/s]'); 
legend('Velocity','Velocity estimate','Location','Northeast');
title('Figure 11.8 (b)');
set(gca,'Fontname','Times New Roman','FontSize',14); % 文字の設定
set(gcf,'color','w');   % 枠の周囲を白く変更する
% 
delete('*.slxc');delete('*.asv');
% 
