%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% プログラム 2.1: 加速度制御に基づくPI速度制御系の例
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;clc;
%% 物理パラメータ
m0=1;d=10;            % 質量公称値[kg])、外乱の大きさ[N]
T=0.01;a=1/T;         % DOBの時定数
zeta=7.5;wn=5;        % 減衰定数、応答周波数
Kp=2*zeta*wn;Ki=wn^2; % PIコントロ－ラ
dt=0.005;d_out=0.001; % ランダムノイズの周期と大きさ

%% シミュレーション
tend=5; % 終了時間
open_system('sim_Figure_2_10_and_2_11_basic_vel_con_PI_DOB');
y=sim('sim_Figure_2_10_and_2_11_basic_vel_con_PI_DOB');

%% 描画
% Simulinkモデルの「f,d＆d_hat」Scopeのログ変数名にd_outを設定しておくこと。
% Simulinkモデルの「Ac_ref＆Ac」Scopeのログ変数名にa_outを設定しておくこと。
% Simulinkモデルの「Vc_ref＆Vc」Scopeのログ変数名にv_outを設定しておくこと。
% Scopeのログの保存形式を「時間付き構造体」に設定する
t=y.d_out.time;
dd=y.d_out.signals(2).values(:,1);
d_hat=y.d_out.signals(2).values(:,2);
figure(1);clf(1);plot(t,dd,'k--',t,d_hat,'k-','LineWidth',1.2);grid on;
xlabel('Time[s]','Fontsize',16);ylabel('Disturbance[N]','Fontsize',12);
axis([0,tend,-10,20]);
legend('Disturbance', 'Est.','Location','Southeast','Fontsize',12);
title('Figure 2.11 (a-1) or (b-1)');
set(gca,'Fontname','Times New Roman','FontSize',12); % 文字の設定
set(gcf,'color','w');  % 枠の周囲を白く変更する
%
a_ref=y.a_out.signals(1).values;ac=y.a_out.signals(2).values;
figure(2);clf(2);plot(t,a_ref,'k--',t,ac,'k-','LineWidth',1.2);grid on;
xlabel('Time[s]','Fontsize',16);ylabel('Acceleration[m/s^2]','Fontsize',12);
axis([0,tend,-20,80]);
legend('Acceleration-ref', 'Acceleration','Location','Northeast','Fontsize',12);
title('Figure 2.11 (a-2) or (b-2)');
set(gca,'Fontname','Times New Roman','FontSize',12); % 文字の設定 
set(gcf,'color','w');   % 枠の周囲を白く変更する
%
vc_ref=y.v_out.signals(1).values;vc=y.v_out.signals(2).values;
figure(3);clf(3);plot(t,vc_ref,'k--',t,vc,'k-','LineWidth',1.2);grid on;
xlabel('Time[s]','Fontsize',16);ylabel('Velocity[m/s]','Fontsize',12);
axis([0,tend,-0.2,1.2]);
legend('Velocity-ref', 'Velocity','Location','Southeast','Fontsize',12);
title('Figure 2.11 (a-3) or (b-3)');
set(gca,'Fontname','Times New Roman','FontSize',12);% 文字の設定
set(gcf,'color','w');   % 枠の周囲を白く変更する

%% その他
delete('*.slxc');delete('*.asv');