%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% プログラム 2.2: 反力推定オブザーバ(RFOB)を用いた反力制御
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;clc;
%% 物理パラメータ
m=1.0;m0=m;c=0.1;k=1; % 質量,粘性摩擦係数,弾性係数
fric=0.25;            % クーロン摩擦の大きさ
fs=  1;               % 最大静止摩擦の大きさ
ref= 2;               % 反力参照値の大きさ
% 静止摩擦モデルの説明は省略しています。 
% プログラムからモデルの仮定を考えてください。

%% DOB, RFOBのパラメータと摩擦モデルのパラメータ
T=0.02;   % DOBのローパスフィルタの時定数 
dt=0.001; % Simulinkの最大ステップサイズ
a1=1/T; a2=a1; % a1は(-1)*DOBの極, a2は(-1)*RFOBの極
delta=[1e-4,1e-3,1e-5,2e-3]; 
% deltaはMATLAB functionの静止摩擦モデル内で用いる閾値 
% delta=[Velocity, acceleration,stop,Coulomb friction]
a3=12;       % a3はストライベック曲線での減衰パラメータ
a4=1e-3;     % a4はクーロン摩擦をシグモイド曲線で表した際のパラメータ
Kp=75;Kd=15; % PD制御ゲイン

%% シミュレーション
t_end=40; % 終了時間
open_system('sim_Figure_2_14_and_2_15_reaction_con_RFOB');
y=sim('sim_Figure_2_14_and_2_15_reaction_con_RFOB');
% Switch_1は静止摩擦の有無を切り替えるためのスイッチ 
% Switch_2はクーロン摩擦の大きさ10%増加の有無を切り替えるスイッチ

%% 描画
% Simulinkモデルの「Reac_ref&f&est」Scopeのログ変数名にreac_f_outを設定しておくこと。
% Simulinkモデルの「Acceleration」Scopeのログ変数名にaoutを設定しておくこと。
% Scopeのログの保存形式を「時間付き構造体」に設定する
t=y.reac_f_out.time;
reac_f_ref=y.reac_f_out.signals(1).values;
reac_f=y.reac_f_out.signals(2).values;
reac_f_est=y.reac_f_out.signals(3).values;
vel=y.vout.signals.values;
f_ref=y.aout.signals(1).values;
f=y.aout.signals(2).values;
a=y.aout.signals(3).values;
%
figure(1);clf(1);
subplot(2,1,1);plot(t,reac_f_ref,'k--',t,reac_f,'k-.',t,reac_f_est,'k-','LineWidth',1.2);grid on;
legend('f reac\_ref','Reac force','f reac\_hat','Location','Northeast');
axis([0,t_end,-1.2*ref,1.2*ref]);
%xlabel('Time[s]','Fontsize',16);
ylabel('Reaction force [N]','Fontsize',16); 
title('Figure 2.15 (a)');
%set(gcf,'color','w');  % 枠の周囲を白く変更する
set(gca,'Fontname','Times New Roman','FontSize',14); % 文字の設定
%
subplot(2,1,2);plot(t,vel,'k-','LineWidth',1.2);grid on;
axis([0,t_end,-1.0,1.0]);
xlabel('Time[s]','Fontsize',16);
ylabel('Velocity v_c[m/s]','Fontsize',16); 
set(gca,'Fontname','Times New Roman','FontSize',14); % 文字の設定
set(gcf,'color','w');  % 枠の周囲を白く変更する
%
figure(2);clf(2);
subplot(2,1,1);plot(t,f_ref,'k--',t,f,'k-','LineWidth',1.2);grid on;
legend('Force-ref(f\_bar)','Real force (f)','Location','Southeast','FontSize',12);
axis([0,t_end,-5,5]);
xlabel('Time[s]','Fontsize',16);
ylabel('Driving force[N]','Fontsize',16);
title('Figure 2.15 (b)');
set(gca,'Fontname','Times New Roman','FontSize',14); % 文字の設定
subplot(2,1,2);plot(t,a,'k-','LineWidth',1.2);grid on;
axis([0,t_end,-5,5]);
xlabel('Time[s]','Fontsize',16);
ylabel('Acceleration[m/s^2]','Fontsize',16); 
set(gcf,'color','w');  % 枠の周囲を白く変更する
set(gca,'Fontname','Times New Roman','FontSize',14); % 文字の設定

%% その他
delete('*.slxc');delete('*.asv');





