%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  プログラム 2.4：図2.27を描画するためのプログラム
% 　　　　　DOB併用PI速度制御における
% 　　　　　質量が変化する場合の影響
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 注意：プログラム 2.4は書籍では省略されています。
%---------------------------------------------------
clear all;clc;clf;close all;format short e;
%% 物理パラメータ
m0=1;c=0; % 質量, 粘性摩擦係数
T=0.01; %1; % DOBのローパスフィルタの時定数 (1sも試すと良い)

% 伝達関数の定義
s=tf('s');
dm=logspace(-0.9,1,10);
[~,size_dm]=size(dm);
for i=1:size_dm
    m=m0*dm(i);P=tf([1],[m,c]);     % Pは制御対象の伝達関数 1/(m*s+c)
    Cob=tf([T,1],[T,0]);            % DOBの正帰還部分に等価な伝達関数 
    % Cobはp.30 図2.19(b)の破線枠上部の数式に相当
    Gob=P*Cob;            % 制御対象Pと上記Cobの直列接続伝達関数
    Hob=tf([m0,0],[T,1]); % DOBの後ろ向き伝達関数。図2.19(b)のフィードバック部分
    Pob=feedback(Gob,Hob);          % DOBを用いた制御対象の見かけ上の伝達関数
    Kp=50;Ki=1;C=tf([Kp,Ki],[1,0]); % PI制御器の伝達関数
    Go=Pob*C;Gc=feedback(Go,1);     % 開ループと閉ループの伝達関数
    [numo,deno]=tfdata(Go); % 開ループ系の伝達関数の分子と分母
    [numc,denc]=tfdata(Gc); % 閉ループ系の伝達関数の分子と分母
  
% 描画
   w=logspace(-1,4,200);% 開ループのボード線図計算 
   figure(1);[gain,phase]=bode(numo,deno,w);
   subplot(2,1,1);semilogx(w,20*log10(gain),'k');hold on;grid on;
   set(gca,'Fontname','Times New Roman','FontSize',14); % 文字の設定
   xlabel('Frequancy [rad/s]');ylabel('Gain[dB]');
   axis([1e-1,1e4,-80,60]);
   title('Figure 2.27 (a-1) or (b-1)');
   subplot(2,1,2);semilogx(w,phase,'k');hold on;grid on;
   axis([1e-1,1e4,-180,0]);
   set(gca,'Fontname','Times New Roman','FontSize',14); % Characters setting
   xlabel('Frequancy [rad/s]','Fontsize',16);ylabel('Phase[deg]','Fontsize',16);
   set(gcf,'color','w');    % 枠の周囲を白く変更する
  %
  figure(2);[polec,zeroc]=pzmap(Gc); % 閉ループ系の極零描画のためのの計算
  plot(real(polec),imag(polec),'*k');hold on;grid on;
  plot(real(zeroc),imag(zeroc),'ok');hold on;
  xlabel('Real','Fontsize',16);ylabel('Imaginary','Fontsize',16);
  axis([-120,20,-50,50]);
  title('Figure 2.27 (a-2) or (b-2)');
  set(gca,'Fontname','Times New Roman','FontSize',14); % 文字の設定
  set(gcf,'color','w');    % 枠の周囲を白く変更する
  %
  tfinal=1.0;
  figure(3);[y,time]=step(Gc,tfinal); % ステップ応答計算
  plot(time,y,'k');hold on;grid on;   
  xlabel('Time[s]','Fontsize',16);ylabel('Velocity[m/s]','Fontsize',16);
  axis([0,1,0,1.5]);
  title('Figure 2.27 (a-3) or (b-3)');
  set(gca,'Fontname','Times New Roman','FontSize',14); % 文字の設定
  set(gcf,'color','w');    % 枠の周囲を白く変更する
end;

delete('*.slxc');delete('*.asv');