%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  プログラム 3.1：既約分解を利用した2自由度制御系の例
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;clc;
%% バネマスダンパ系の物理パラメータと数学モデルの例 
m=1;c=3;k=2;A=[0,1;-k/m,-c/m];B=[0;1/m];C=[1,0];
[num,den]=ss2tf(A,B,C,0);P=tf(num,den);[z,p,k]=tf2zp(num,den);

%% 制御系設計とオブザーバ設計
%　具体的な設計法は第4章で紹介しています。
pole=[-2,-3];F=place(A,B,pole);Af=A-B*F;
pole_ob=[-3,-5];H_temp=place(A',C',pole_ob);H=H_temp';Ah=A-H*C;

%% P_nd=N*inv(D): 3.1.1 (1)の作成
[num_D,den_D]=ss2tf(Af,B,-F,1);D=tf(num_D,den_D); % 式(3.4)
[num_N,den_N]=ss2tf(Af,B,C,0);N=tf(num_N,den_N);  % 式(3.5)
P_nd=N*inv(D);
[n_nd,d_nd]=tfdata(P_nd,'v');[z_nd,p_nd,k_nd]=tf2zp(n_nd,d_nd);

%% tild_P=inv(tilde_D)*tilde_N: 3.1.1 (2)の作成
[num_tN,den_tN]=ss2tf(Ah,B,C,0);tilde_N=tf(num_tN,den_tN); % 式(3.8)
[num_tD,den_tD]=ss2tf(Ah,H,-C,1);tilde_D=tf(num_tD,den_tD);% 式(3.11)
P_tilde=inv(tilde_D)*tilde_N;[n_til,d_til]=tfdata(P_tilde,'v');
[z_til,p_til,k_til]=tf2zp(n_til,d_til);

%% 制御器と二重既約分解の確認
% XとYの作成:　3.1.1 (3)
[num_X,den_X]=ss2tf(Af,H,C,1);X=tf(num_X,den_X);% 式(3.18)の前
[z_X,p_X,k_X]=tf2zp(num_X,den_X);
[num_Y,den_Y]=ss2tf(Af,H,F,0);Y=tf(num_Y,den_Y);% 式(3.17)
[z_Y,p_Y,k_Y]=tf2zp(num_Y,den_Y);
%
C_XY=Y*inv(X);[num_XY,den_XY]=tfdata(C_XY,'v');
[z_C_XY,p_C_XY,k_C_XY]=tf2zp(num_XY,den_XY);

%% tilde_Xとtilde_Yの作成
[num_til_X,den_til_X]=ss2tf(Ah,B,F,1);tilde_X=tf(num_til_X,den_til_X);% 式(3.14)
[z_til_X,p_til_X,k_til_X]=tf2zp(num_til_X,den_til_X);
[num_til_Y,den_til_Y]=ss2tf(Ah,H,F,0);tilde_Y=tf(num_til_Y,den_til_Y);% 式(3.15)
[z_til_Y,p_til_Y,k_til_Y]=tf2zp(num_til_Y,den_til_Y);

%% Q(s)とK(s)の設計：3.3節に従って
T=0.05;num_H=1;den_H=[T,1];H=tf(num_H,den_H);
num_Q=conv(conv(num_til_X,num_H),den_tN);
num_tN_2=conv(num_tN,[0.01,1]);
den_Q=conv(conv(den_til_X,den_H),num_tN_2);
%
zeta=1;w=10; % Kのプロパー化のためのパラメータ
num_K=den_N*w^2;den_K=conv(num_N,[1,2*zeta*w,w^2]);

%% シミュレーション
open_system('sim_Figure_3_6_and_3_7_yula_Qcon');
z=sim('sim_Figure_3_6_and_3_7_yula_Qcon');

%% 描画
% Simulinkモデルの「d&d_hat」Scopeのログ変数名にd_outを設定しておくこと。
% Simulinkモデルの「r&y」Scopeのログ変数名にy_outを設定しておくこと。
% Scopeのログの保存形式を「時間付き構造体」に設定する
t=z.y_out.time;
r=z.y_out.signals(1).values;y=z.y_out.signals(2).values;
d=z.d_out.signals(1).values;d_hat=z.d_out.signals(2).values;
%
figure(1);clf(1);
plot(t,d,'--k',t,d_hat,'-k','LineWidth',1.2);grid on;
xlabel('Time [s]','Fontsize',16);ylabel('Disturbance [N]','Fontsize',16); 
axis([0,10,0,1.2]);  
legend('Disturbance','Disturbance Estimate','Location','Southeast','Fontsize',16) 
title('Figure 3.7 (a)');
set(gca,'Fontname','Times New Roman','FontSize',14); % 文字の設定
set(gcf,'color','w');   % 枠の周囲を白く変更する
%
figure(2);clf(2);
plot(t,r,'--k',t,y,'-k','LineWidth',1.2);grid on;
xlabel('Time [s]','Fontsize',16);ylabel('Position Xc [m]','Fontsize',16);
axis([0,10,0,1.2]);
legend('Position Reference','Real Position','Location','Southeast','Fontsize',16)
title('Figure 3.7 (b)');
set(gca,'Fontname','Times New Roman','FontSize',14); % 文字の設定
set(gcf,'color','w');   % 枠の周囲を白く変更する

delete('*.slxc');delete('*.asv');






