%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  プログラム 4.1: 同一次元入力端外乱オブザーバ(速度観測型)の例
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;clc;
%% 物理パラメータ
m=2;   % 質量[kg]
dis_step=1.0;
%% 状態空間モデル
A=0;B=1/m;C=1;% 連続系のA,B,C行列
%% 拡大系の設計とオブザーバゲインの設計
Abar=[A,-B;0,0];Bbar=[B;0];Cbar=[C,0];
Ov=[Cbar;Cbar*Abar;Cbar*Abar^2];rank_of_Ov=rank(Ov);% Evaluate rank
pole_ob=[-10,-12]; % DOBの極の指定
Htemp=place(Abar',Cbar',pole_ob);H=Htemp';H1=H(1);H2=H(2);

%% シミュレーション
x0=[0.0];tend=5.0; % 初期速度と終了時間
open_system('sim_Figure_4_4_and_4_5_dob_input_dis_vel_out');
set_param('sim_Figure_4_4_and_4_5_dob_input_dis_vel_out','WideLines','on'); 
set_param('sim_Figure_4_4_and_4_5_dob_input_dis_vel_out','ShowLineDimensions','on');
z=sim('sim_Figure_4_4_and_4_5_dob_input_dis_vel_out'); 

%% 描画
% SimulinkモデルのScope'y and y_hat'のログでyoutを指定する
% Scopeの'd and d_hat'のログでdoutを指定する
% Scopeのログの保存形式を「時間付き構造体」に設定する
t=z.yout.time; 
d=z.dout.signals(1).values;
d_est=z.dout.signals(2).values;
vc=z.yout.signals(1).values;vc_hat=z.yout.signals(2).values;
%
figure(1);clf(1);
plot(t,d,'k--',t,d_est,'k','LineWidth',1.2);grid on;
axis([0,tend,-0.2,1.4]);
legend('Real Disturbance','Disturbanse Estimate','Location','Northeast');
xlabel('Time[s]');ylabel('Disturbance and Estimate [N]'); 
title('Figure 4.5(a)')
set(gca,'Fontname','Times New Roman','FontSize',14); % 文字の設定
set(gcf,'color','w');   % 枠の周囲を白く変更する
%
figure(2);clf(2);
plot(t,vc,'k--',t,vc_hat,'k','LineWidth',1.2);grid on;
axis([0,tend,-0.1,0.04]);
xlabel('Time[s]');ylabel('Velocity Vc and Estimate[m/s]'); 
legend('Real Velocity','Velocity Estimate','Location','Northeast');
title('Figure 4.5(b)')
set(gca,'Fontname','Times New Roman','FontSize',14); % 文字の設定
set(gcf,'color','w');   % 枠の周囲を白く変更する

delete('*.slxc');




