%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% プログラム 4.2: 同一次元入力端外乱オブザーバ(位置観測型)の例
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;clc;
%% 物理パラメータ
m=2;            % 質量[kg]
dis_step=10.0;  % 外乱の大きさ[N]
%% 状態空間モデル
A=[0,1;0,0];B=[0;1/m];C=[1 0]; %連続系のA,B,C行列

%% 拡大系の設計とオブザーバゲインの設計
Abar=[A,-B;zeros(1,2),0];Bbar=[B;0];Cbar=[C,0];      % 拡大系の作成
Ov=[Cbar;Cbar*Abar;Cbar*Abar^2];rank_of_Ov=rank(Ov); % ランクの確認
pole_dob=[-13,-14,-15]*5;   % DOBの極の指定 （*5は後から調整のために追加)
Htemp=place(Abar',Cbar',pole_dob);H=Htemp';H1=H(1:2);H2=H(3);

%% LQ制御法を用いた状態フィードバックゲインの設計
Q=diag([1e3,1e1]);R=1e2;F=lqr(A,B,Q,R);

% シミュレーション
x0=[0.1;0.0];tend=10.0; % 位置・速度の初期値と終了時間
open('sim_Figure_4_6_and_4_7_dob_input_dis_pos_out'); 
set_param('sim_Figure_4_6_and_4_7_dob_input_dis_pos_out','WideLines','on'); 
set_param('sim_Figure_4_6_and_4_7_dob_input_dis_pos_out','ShowLineDimensions','on');
y=sim('sim_Figure_4_6_and_4_7_dob_input_dis_pos_out');

%% DOBと比較するための一般的なオブザーバの設計と実行
% 比較結果は図4.8 
pole_ob=[-13,-15]*5;
H_temp_=place(A',C',pole_ob);Hnormal=H_temp_';
open_system('sim_Figure_4_8_ob_input_dis_pos_out');
z=sim('sim_Figure_4_8_ob_input_dis_pos_out');

%% 描画
% 'sim_Figure_4_6_and_4_7_dob_input_dis_pos_out'のScope'd&d_hat'のログでd_outを指定
% 同じくScope'Vc&Vc_hat'のログでvc_outを、'XC&XC_hat'でxc_outを指定する
% 'sim_Figure_4_8_ob_input_dis_pos_out'のScope'Vc＆Vchat'のログでvcn_outを指定
% 同じくScope'XC&XC_hat'でxcn_outを指定する
% Scopeのログの保存形式を「時間付き構造体」に設定する
t=y.d_out.time;
d =y.d_out.signals(1).values;
d_hat=y.d_out.signals(2).values;
vc=y.vc_out.signals(1).values;
vc_hat=y.vc_out.signals(2).values;
xc=y.xc_out.signals(1).values;
xc_hat=y.xc_out.signals(2).values;
%
figure(1);clf(1);
plot(t,d,'k--',t,d_hat,'k','LineWidth',1.2);grid on;
axis([0,tend,-2,12]);
legend('Disturbance','Disturbanse Estimate','Location','Southeast');
xlabel('Time[s]');ylabel('Disturbance and Estimate [N]'); 
title('Figure 4.7 (a-1)');
set(gca,'Fontname','Times New Roman','FontSize',14); %% 文字の設定
set(gcf,'color','w');   % 枠の周囲を白く変更する
%
figure(2);clf(2);
plot(t,vc,'k--',t,vc_hat,'k','LineWidth',1.2);grid on;
axis([0,tend,-2.2,0.5]);
xlabel('Time[s]');ylabel('Velocity Vc and Estimate[m/s]'); 
legend('Real velocity','Velocity Estimate','Location','Southeast');
title('Figure 4.7 (a-2)');
set(gca,'Fontname','Times New Roman','FontSize',14); % 文字の設定
set(gcf,'color','w');   % 枠の周囲を白く変更する
%
figure(3);clf(3);
plot(t,xc,'k--',t,xc_hat,'k','LineWidth',1.2);grid on;
axis([0,tend,-4,1]);
xlabel('Time[s]');ylabel('Position Xc and Estimate[m]'); 
legend('Real position','Position Estimate','Location','Southeast');
title('Figure 4.7 (a-3');
set(gca,'Fontname','Times New Roman','FontSize',14); % 文字の設定
set(gcf,'color','w');  % 枠の周囲を白く変更する

%% 一般的なオブザーバ併用制御系の描画
tn=z.xcn_out.time;
vcn=z.vcn_out.signals(1).values;
vcn_hat=z.vcn_out.signals(2).values;
xcn=z.xcn_out.signals(1).values;
xcn_hat=z.xcn_out.signals(2).values;
%
figure(4);clf(4);
plot(tn,vcn,'k--',tn,vcn_hat,'k','LineWidth',1.2);grid on;
axis([0,tend,-2.2,0.5]);
xlabel('Time[s]');ylabel('Velocity Vc and Estimate[m/s]'); 
legend('Real velocity','Velocity Estimate','Location','Southeast');
title('Figure 4.8 (a)');
set(gca,'Fontname','Times New Roman','FontSize',14); % 文字の設定
set(gcf,'color','w');   % 枠の周囲を白く変更する
%
figure(5);clf(5); % 図4.8(b)
plot(tn,xcn,'k--',tn,xcn_hat,'k','LineWidth',1.2);grid on;
axis([0,tend,-4,1]);
xlabel('Time[s]');ylabel('Position Xc and Estimate[m]'); 
legend('Real position','Position Estimate','Location','Northeast');
title('Figure 4.8 (b)');
set(gca,'Fontname','Times New Roman','FontSize',14); % 文字の設定
set(gcf,'color','w');   % 枠の周囲を白く変更する

delete('*.slxc');delete('*.asv');


