%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% プログラム 4.3: 同一次元外乱オブザーバ併用サーボ系(位置観測型)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;clc;
%% 物理パラメータ
m=2;           % 質量[kg]
dis_step=10.0; % 外乱の大きさ[N]
%% 状態空間モデル
A=[0,1;0,0];B=[0;1/m];C=[1 0];% A,B,C matrices

%% 同一次元外乱オブザーバの設計
Abar=[A,-B;zeros(1,2),0];Bbar=[B;0];Cbar=[C,0];      % 拡大系の作成
Ov=[Cbar;Cbar*Abar;Cbar*Abar^2];rank_of_Ov=rank(Ov); % ランクの確認
pole_dob=[-13,-14,-15]*5;                            % DOBの極の設定
Htemp=place(Abar',Cbar',pole_dob);H=Htemp';H1=H(1:2);H2=H(3);

%% サーボ系設計
Atilde=[A,zeros(2,1);-C,0];Btilde=[B;0];
Q=diag([3e5,3e3,2e6]);R=1e0;Ftilde=lqr(Atilde,Btilde,Q,R);
F=Ftilde(1:2);Ki=-Ftilde(3);

%% シミュレーション
x0=[0.0;0.0];tend=10.0;Xc_ref=1.0; % 位置と速度の初期値、終了時間、位置参照値
open_system('sim_Figure_4_9_and_4_10_servo_with_dob_pos_out');
set_param('sim_Figure_4_9_and_4_10_servo_with_dob_pos_out','WideLines','on'); 
set_param('sim_Figure_4_9_and_4_10_servo_with_dob_pos_out','ShowLineDimensions','on');
z=sim('sim_Figure_4_9_and_4_10_servo_with_dob_pos_out'); % 実行

%% 描画
% SimulinkモデルのScope'd&d_hat'のログでdoutを指定する
% Scope'Xc&XC_hat'のログでxcoutを,'Vc&VC_hat'のログでvcoutを指定する
% Scopeのログの保存形式を「時間付き構造体」に設定する
time=z.dout.time;
d=z.dout.signals(1).values;d_est=z.dout.signals(2).values;
xc_ref=z.xcout.signals(1).values;
xc_dob=z.xcout.signals(2).values;
xc_hat_dob=z.xcout.signals(3).values;
vc_dob=z.vcout.signals(1).values;
vc_hat_dob=z.vcout.signals(2).values;
%
figure(1);clf(1); % 外乱推定の評価
plot(time,d,'k--',time,d_est,'k','LineWidth',1.2);grid on;
axis([0,tend,-2,12]);
legend(' Disturbance',' Dis.Estimate','Location','Southeast');
xlabel('Time[s]');ylabel('Disturbance [N]'); 
title('Figure 4.10 (a)');
set(gca,'Fontname','Times New Roman','FontSize',12); % 文字の設定
set(gcf,'color','w');   % 枠の周囲を白く変更する
%
figure(2);clf(2); % 速度と速度推定値の評価
plot(time,vc_dob,'k--',time,vc_hat_dob,'k','LineWidth',1.2);grid on;
axis([0,tend,-0.25,2.0]);
xlabel('Time[s]');ylabel('Velocity and Estimate[m/s]'); 
legend(' Velocity',' Velocity Estimate','Location','Northeast');
title('Figure 4.10 (b)');
set(gca,'Fontname','Times New Roman','FontSize',12); % 文字の設定 
set(gcf,'color','w');   % 枠の周囲を白く変更する
%
figure(3);clf(3); % 位置と位置推定値の評価
plot(time,xc_ref,'k--',time,xc_dob,'k-',time,xc_hat_dob,'k-.','LineWidth',1.2);grid on;
axis([0,tend,-0.2,1.2]);
xlabel('Time[s]');ylabel('Position and Estimate[m]'); 
%legend('Position','Position Estimate','Location','Northeast');
legend('PositinReference',' Position',' Position Estimate','Location','Southeast');
title('Figure 4.10 (c)');
set(gca,'Fontname','Times New Roman','FontSize',12); % 文字の設定 
set(gcf,'color','w');   % 枠の周囲を白く変更する
%% その他
delete('*.slxc');delete('*.asv');




