%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% プログラム 4.4: 同一次元反力推定オブザーバRFOB
%   　　　　　 (位置観測型)の例
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;clc;
%% 物理パラメータ
m=2;c=0.5; % 質量[kg]と粘性摩擦係数[N/(m/s)]
k=15.0;fc=3.0;% 弾性係数[N/m]とクーロン摩擦の大きさ[N]
%% 状態空間モデル
A=[0,1;0,0];B=[0;1/m];C=[1 0]; % 弾性係数kと粘性係数cを無視したA,B,C行列の作成 

%% 拡大系の作成とオブザーバ設計
Abar=[A,-B;zeros(1,2),0];Bbar=[B;0];Cbar=[C,0];
Ov=[Cbar;Cbar*Abar;Cbar*Abar^2];rank_of_Ov=rank(Ov);% ランクの確認
% DOB設計
pole_ob=[-20,-22,-24];    % DOBの極
Htemp=place(Abar',Cbar',pole_ob);H=Htemp';H1=H(1:2);H2=H(3);
% RFOB設計
pole_rfob=[-30,-32,-34];  % RFOBの極
Hrtemp=place(Abar',Cbar',pole_rfob);Hr=Hrtemp';Hr1=Hr(1:2);Hr2=Hr(3);

%% 状態フィードバック制御系設計
pole=[-3,-4];F=place(A,B,pole); %極配置法による設計

%% シミュレーション
x0=[0.1;0.0];tend=5.0; % 位置と速度の初期値, 終了時間
open_system('sim_Figure_4_11_and_4_12_contenious_RFOB_pos');
set_param('sim_Figure_4_11_and_4_12_contenious_RFOB_pos','WideLines','on'); 
set_param('sim_Figure_4_11_and_4_12_contenious_RFOB_pos','ShowLineDimensions','on');
z=sim('sim_Figure_4_11_and_4_12_contenious_RFOB_pos'); 

%% 描画
% SimulinkモデルのScope'Reac_f＆Reac_f_hat'のログ変数名をd_estimateに, 
% Scope'x & x_hat'のログの変数名にx.outを設定する
% Scopeのログの保存形式を「時間付き構造体」に設定する
time=z.d_estimate.time;d=z.d_estimate.signals(1).values;
d_est=z.d_estimate.signals(2).values;
vc=z.x_out.signals(1).values(:,2);
vc_hat=z.x_out.signals(2).values(:,2);
%
figure(1);clf(1); % 反力と反力推定値の評価
plot(time,d,'k--',time,d_est,'k','LineWidth',1.2);grid on;
axis([0,tend,-0.25,1.75]);
legend('Reaction Force','Reaction F.Estimate','Location','Northeast');
xlabel('Time[s]');ylabel('Reaction Force & Estimate [N]'); 
title('Figure 4.12 (a)');
set(gca,'Fontname','Times New Roman','FontSize',12); % 文字の設定
set(gcf,'color','w');   % 枠の周囲を白く変更する
%
figure(2);clf(2); % 速度と速度推定値の評価
plot(time,vc,'k--',time,vc_hat,'k','LineWidth',1.2);grid on;
axis([0,tend,-0.15,0.05]);
xlabel('Time[s]');ylabel('Velocity & Estimate[m/s]'); 
legend('Velocity','Velocity Estimate','Location','Southeast');
title('Figure 4.12 (b)');
set(gca,'Fontname','Times New Roman','FontSize',12); % 文字の設定
set(gcf,'color','w');   % 枠の周囲を白く変更するr

delete('*.slxc');delete('*.asv');


