%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  プログラム 4.5: 同一次元出力端外乱オブザーバの例
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;clc;
%% 物理パラメータ
m=2;   % 質量[kg]
k=3;   % 弾性係数[N/m]
dis_step_xc=0.1; % 出力端外乱の大きさ[m]
%% 状態空間モデル
A=[0,1;-k/m,0];B=[0;1/m];C=[1 0];% A,B,C matrices

%% 拡大系の作成と出力端外乱オブザーバの設計
Abar=[A,zeros(2,1);zeros(1,2),0];
Bbar=[B;0];Cbar=[C,-1];
Ov=[Cbar;Cbar*Abar;Cbar*Abar^2];rank_of_Ov=rank(Ov) % ランクの確認
Q_ob=diag([1e1,1e1,1e5]);R_ob=1;  % DOB設計のためのQとRの設定
Htemp=lqr(Abar',Cbar',Q_ob,R_ob); % lqr関数を用いたDOBゲイン設計
H=Htemp';Ho1=H(1:2,:);Ho2=H(3,:);

%% 状態フィードバック制御系設計
pole=[-1,-1.5];F=place(A,B,pole); % 極配置法による設計

%% シミュレーション
x0=[1.0;0.0];tend=10;  % 位置と速度の初期値, 終了時間
open('sim_Figure_4_13_and_4_14_Output_DOB');
set_param('sim_Figure_4_13_and_4_14_Output_DOB','WideLines','on'); 
set_param('sim_Figure_4_13_and_4_14_Output_DOB','ShowLineDimensions','on');
z=sim('sim_Figure_4_13_and_4_14_Output_DOB'); 

%% 描画
% SimulinkモデルのScope'd＆and d_hat'のログ変数名をdoutに, 
% Scope'Xc & Xc_hat'のログの変数名にxcoutを設定する
% Scopeのログの保存形式を「時間付き構造体」に設定する
time=z.dout.time;
d=z.dout.signals(1).values;
d_est=z.dout.signals(2).values;
xc=z.xcout.signals(1).values;
xc_hat=z.xcout.signals(2).values;
%
figure(1);clf(1); % 出力端外乱と出力端外乱推定値の評価
plot(time,d,'k--',time,d_est,'k','LineWidth',1.2);grid on;
axis([0,tend,-0.05,0.2]);
legend('Output Disturbance','Disturbanse Estimate','Location','Northeast');
xlabel('Time[s]');ylabel('Disturbance and Estimate [N]'); 
title('Figure 4.14 (a)');
set(gca,'Fontname','Times New Roman','FontSize',12);  % 文字の設定 
set(gcf,'color','w');    % 枠の周囲を白く変更する
%
figure(2);clf(2); % 位置と位置推定値の評価
plot(time,xc,'k--',time,xc_hat,'k','LineWidth',1.2);grid on;
axis([0,tend,0,1.2]);
xlabel('Time[s]');ylabel('Position Xc and Estimate[m]'); 
legend('Real Position','Position Estimate','Location','Northeast');
title('Figure 4.14 (b)');
set(gca,'Fontname','Times New Roman','FontSize',12);  % 文字の設定
set(gcf,'color','w');    % 枠の周囲を白く変更する

delete('*.slxc');delete('*.asv');



