%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% プログラム 4.6: 同一次元高次外乱オブザーバの例
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;clc;
%% 物理パラメータ
m=1.0;c=2.0;  % 質量[kg]と粘性摩擦係数[N/(m/s)]
A=[0 1; 0 -c/m];B=[0;1/m];C=[1 0];
[n,m]=size(B); % 制御対象の次数と入力数

%% 状態フィードバック制御系設計
pole=[-2.0,-3.0];F=place(A,B,pole); % 極配置法による設計

%% p次元外乱と評価のための設定 
p=1;          % 外乱の次元を設計者が決める(例えば,p=0～4は可観測とする)
tend=5;       % 終了時間
dis_step=1.0; % ステップ状外乱の大きさ

%% p次元DOB設計のための設定および拡大系の作成
N=B; % マッチング条件を満たすと仮定
H=1;Gamma=zeros(p+1,p+1);for i=1:p H=[H,0];end;
Gamma(1:p,2:p+1)=eye(p);% Gammaの作成
Abar=[A,-N*H;zeros(p+1,n),Gamma];Bbar=[B;zeros(p+1,1)];
Cbar=[C,zeros(1,p+1)];

%% p次元DOB設計  
pole_ob=[];base_of_pole=-15;delta_pole=2;
for i=1:n+p+1 pole_ob=[pole_ob,base_of_pole-delta_pole*i]; end;
Htemp=place(Abar',Cbar',pole_ob);Hob=Htemp';% 極配置法でのゲイン設計
H1=Hob(1:n);H2=Hob(n+1:n+p+1);% 状態推定ゲインH1と外乱推定ゲインH2   

%% シミュレーション
open('sim_Figure_4_15_and_4_16_higher_order_DOB');
x0=[0.05;0]; % 位置と速度の初期値
set_param('sim_Figure_4_15_and_4_16_higher_order_DOB','WideLines','on'); 
set_param('sim_Figure_4_15_and_4_16_higher_order_DOB','ShowLineDimensions','on')
z=sim('sim_Figure_4_15_and_4_16_higher_order_DOB');
% 注意：初期値が既知か未知かをSimulinkのManual Switch1で切り替える

%% 描画
% SimulinkモデルのScope'd＆d_hat'のログ変数名にdoutを設定しておく
% Scopeのログの保存形式を「時間付き構造体」に設定する
time=z.dout.time;
d=z.dout.signals(1).values;
d_est=z.dout.signals(2).values;
%
plot(time,d,'k--',time,d_est,'k-','LineWidth',1.2);grid on;
legend('Disturbance','Disturbanse Estimate','Location','Southeast');
xlabel('Time[s]','Fontsize',16);
ylabel('Disturbance and Estimate','Fontsize',16); 
axis([0,tend,-1,5]);
title('Figure 4.16');
set(gca,'Fontname','Times New Roman','FontSize',14); % 文字の設定
set(gcf,'color','w');   % 枠の周囲を白く変更する

delete('*.slxc');delete('*.asv');
