%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% プログラム 4.7: ランプ状(1次)最小次元外乱オブザーバの設計
%                (速度観測型)の例
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;clc;
%% 物理パラメータ
m=2.0; % 質量[kg]

%% 拡大系の状態空間モデルの作成
A=0;B=1/m;C=1;D=0;
A_bar=[A -B,0;zeros(2,2),[1;0]];
A11=A_bar(1,1);A12=A_bar(1,2:3);A21=A_bar(2:3,1);A22=A_bar(2:3,2:3);
B_bar=[B;zeros(2,1)];B1=B_bar(1);B2=B_bar(2:3);
C_bar=[C,zeros(1,2)];D_bar=0;

%% ランプ状(1次)最小次元外乱オブザーバの設計
p1=-8;p2=-9; % オブザーバの極
g1=-(p1+p2);g2=p1*p2; % 多項式の係数
% 注意: A_hatの特性多項式は's^2+g1s+g2'となる
L1=-m*g1;L2=-m*g2;L=[L1;L2];
A_hat=A22-L*A12;B_hat=A_hat*L+A21-L*A11;
C_hat=[zeros(1,2);eye(2)];D_hat=[1;L];J_hat=-L*B1+B2;

%% LQ制御法を用いた制御系設計
Q=1.5;R=2;F=lqr(A,B,Q,R);

%% シミュレーション
vc0=0;d=1.0;t_end=5.0; % 速度の初期値, 外乱の大きさ, 終了時間
open_system('sim_Figure_4_23_and_4_24_min_1order_DOB');
set_param('sim_Figure_4_23_and_4_24_min_1order_DOB','WideLines','on'); 
set_param('sim_Figure_4_23_and_4_24_min_1order_DOB','ShowLineDimensions','on')
y=sim('sim_Figure_4_23_and_4_24_min_1order_DOB');

%% 描画
% SimulinkモデルのScope'd＆_hat_d'のログ変数名にd_outを設定しておく
% Scopeのログの保存形式を「時間付き構造体」に設定する
t=y.d_out.time;
d=y.d_out.signals(1).values;
d_hat=y.d_out.signals(2).values;
% %
figure(1);clf(1);plot(t,d,'k--',t,d_hat,'k-','LineWidth',1.2);grid on;
legend('Disturbance','Disturbanse Estimate','Location','Northeast');
axis([0,t_end,-0.25,1.5]);xlabel('Time[s]','Fontsize',16);
ylabel('Disturbance[N]','Fontsize',16); 
title('Figure 4.24');
set(gca,'Fontname','Times New Roman','FontSize',14);  % 文字の設定
set(gcf,'color','w');   % 枠の周囲を白く変更する


delete('*.asv');delete('*.slxc');