%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% プログラム 4.8: 周期外乱オブザーバ(位置観測型)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;clc;%close all;
%% 物理パラメータ
m=2;             % 質量[kg]
dis_step=1;      % ステップ状外乱の大きさ
w=2*pi*1;a=1;    % DOB設計用の正弦波外乱の角周波数wと大きさa 
Aw=[0,1;-w^2,0];Bw=[0;1];Cw=[a*w,0];
wex=w*1;aex=a*1; % 実外乱の角周波数と振幅(必ずしも一致しない)

%% 状態空間モデル
A=[0,1;0,0];B=[0;1/m];C=[1 0];% ABC matrix of continuous system

%% 拡大系の作成とオブザーバゲインの設計
Abar=[A,-B*Cw;zeros(2,2),Aw]; 
Bbar=[B;zeros(2,1)];Cbar=[C,zeros(1,2)];
pole_ob=[-60,-65,-70,-75]*0.2;  % 極配置法での設計
Htemp=place(Abar',Cbar',pole_ob);H=Htemp';
H1=H(1:2);H2=H(3:4);  % H1は状態推定ゲイン, H2は外乱推定用ゲイン

%% 極配置法に夜状態フィードバックゲインの設計
pole=[-3,-4];F=place(A,B,pole);

%% シミュレーション
x0=[0.0];tend=5; % 位置と速度の初期値, 終了時間
amp_pulse=1;Ts=10;puls_width=10/amp_pulse;
open_system('sim_Figure_4_25_and_4_26_periodic_dob');
set_param('sim_Figure_4_25_and_4_26_periodic_dob','WideLines','on'); 
set_param('sim_Figure_4_25_and_4_26_periodic_dob','ShowLineDimensions','on');
z=sim('sim_Figure_4_25_and_4_26_periodic_dob');  

%% 描画
% SimulinkモデルのScope'd and d_hat'のログ変数名にdoutを設定しておく
% SimulinkモデルのScope'Xc and Xc_hat'のログ変数名にxoutを設定しておく
% Scopeのログの保存形式を「時間付き構造体」に設定する
time=z.dout.time;
d=z.dout.signals(1).values;
d_est=z.dout.signals(2).values;
xc=z.xout.signals(1).values;
xc_hat=z.xout.signals(2).values;
%
figure(1);clf(1); % 外乱と外乱推定値の描画
plot(time,d,'k--',time,d_est,'k','LineWidth',1.2);grid on;
axis([0,tend,-aex*1.2,aex*1.7]);
legend('Disturbance','Disturbanse Estimate','Location','Northeast');
xlabel('Time[s]');ylabel('Disturbance and Estimate [N]'); 
title('Figure 4.26 (a)');
set(gca,'Fontname','Times New Roman','FontSize',14);  % 文字の設定
set(gcf,'color','w');   % 枠の周囲を白く変更する
%
figure(2);clf(2); % 位置と位置推定値の描画
plot(time,xc,'k--',time,xc_hat,'k','LineWidth',1.2);grid on;
axis([0,tend,-0.08,0.02]);
xlabel('Time[s]');ylabel('Position Xc and Estimate[m]'); 
legend('Real position','Position Estimate','Location','Northeast');
title('Figure 4.26 (b)');
set(gca,'Fontname','Times New Roman','FontSize',14);  % 文字の設定
set(gcf,'color','w');   % 枠の周囲を白く変更する

delete('*.slxc');delete('*.asv');