%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% プログラム 4.9 DCモータの非入出力端外乱推定の例
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;clc;
%% 物理パラメータ
Ra=0.65;La=5e-2; % 電機子抵抗 & 電機子インダクタンス
Ke=2.8e-2; % 誘導起電力定数
Kt=Ke;     % トルク定数 
J=1e-1;    % ロータの慣性モーメント
k=0;       % モータに接続するねじりバネの弾性係数
% 図4.27に対応。一般にはねじりバネは搭載しないので0値。

%% 状態空間モデル
% 状態変数x=[theta(t);omega(t);i(t)],制御入力u=va(t)とする
A=[0,    1,      0;
   -k/J, 0,      Kt/J;
   0,    -Ke/La, -Ra/La];
B=[0;0;1/La];Bdis=[0;1/J;0];C1=[1 0 0];D=0;

%% 拡大系の作成とオブザーバ設計
A_=[A -Bdis;zeros(1,4)];B_=[B;0];C1_=[C1 0];D_=D;
pole_ob=[-5,-6,-7,-8];H_temp=place(A_',C1_',pole_ob);
H=H_temp';H1=H(1:3);H2=H(4);

%% 安定化制御系設計
pole_fb=[-1.2,-1.3,-1.4];F=place(A,B,pole_fb);

%% シミュレーション
open('sim_Figure_4_29_and_4_30_for_dc_motordc_motor');
t_end=20;x0=[0;0;0];tau_d=1; % tau_d：外乱トルクの大きさ
set_param('sim_Figure_4_29_and_4_30_for_dc_motordc_motor','WideLines','on'); 
set_param('sim_Figure_4_29_and_4_30_for_dc_motordc_motor','ShowLineDimensions','on');
z=sim('sim_Figure_4_29_and_4_30_for_dc_motordc_motor');

%% 描画
% SimulinkモデルのScope'd&d_hat'のログでd_outを指定する
% Scope'y&y_hat'のログでy_outを指定する
% Scopeのログの保存形式を「時間付き構造体」に設定する
t=z.d_out.time;
d=z.d_out.signals(1).values;
d_hat=z.d_out.signals(2).values;
figure(1);clf(1); % 外乱＆外乱推定値の描画
plot(t,d,'--k',t,d_hat,'k','LineWidth',1.2);grid on;
axis([0,t_end,-0.2,1.2]);
xlabel('Time [s]','Fontsize',16);ylabel('Disturbance torque[Nm]','Fontsize',16); 
legend('Disturbance torque','Disturbance estimate','Location','Southeast');
title('Figure 4.30(a)');
set(gca,'Fontname','Times New Roman','FontSize',14); % 文字の設定
set(gcf,'color','w');  % 枠の周囲を白く変更する

theta=z.y_out.signals(1).values;
theta_hat=z.y_out.signals(2).values;
figure(2);clf(2); % 回転角度＆回転角度推定値の描画
plot(t,theta,'--k',t,theta_hat,'k','LineWidth',1.2);grid on;
axis([0,t_end,-20,5]);
xlabel('Time [s]','Fontsize',16);ylabel('Angle[rad]','Fontsize',16); 
legend('Theta','Theta-estimate');
title('Figure 4.30(b)');
set(gca,'Fontname','Times New Roman','FontSize',14); % 文字の設定
set(gcf,'color','w');   % 枠の周囲を白く変更する

%% 
delete('*.slxc');delete('*.asv');