%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  プログラム 5.1: 同一次元ディジタルDOB
%          (速度観測型)の例
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;clc;
%% 物理パラメータの定義
m=1.0; % 質量[kg]

%% 拡大系の状態空間モデルの作成
A=[0 -1/m;0,0];B=[1/m;0];C=[1,0];D=0;
[n,~]=size(A);[~,m]=size(B);[p,~]=size(C);
T=0.01;[Ad,Bd,Cd,Dd]=c2dm(A,B,C,D,T); % c2dmを用いたディジタル化
% or Ad=[0,-T/m;0,1],Bd=[T/m;0]　%　この例では左記をもちたも良い

%% 外乱オブザーバ設計
pole_ob=[0.87,0.92]; % 絶対値1未満の極の指定
H=place(Ad',Cd',pole_ob);H1=H(1);H2=H(2);

%% シミュレーション
f=0.5;d=1.0;t_end=5.0; % 駆動力,外乱の大きさ,終了時間
open_system('sim_Figure_5_4_and_5_5_identity_digital_dob_with_vel');
y=sim('sim_Figure_5_4_and_5_5_identity_digital_dob_with_vel');

%% 描画
% SimulinkモデルのScope'd＆d_hat'のログ変数名にd_dataを設定しておく
% SimulinkモデルのScope'Vc＆hat_Vc'のログ変数名にy_dataを設定しておく
% Scopeのログの保存形式を「時間付き構造体」に設定する
t=y.y_data.time;t2=y.d_data.time;
vc=y.y_data.signals(1).values;
hat_vc=y.y_data.signals(2).values;
d=y.d_data.signals(1).values;
hat_d=y.d_data.signals(2).values;
%
figure(1);clf(1); %　外乱と外乱推定値の描画
plot(t2,d,'k--',t2,hat_d,'k-','LineWidth',1.2);grid on;
legend('Disturbance','Disturbanse Estimate','Location','Northeast');
axis([0,t_end,-0.2,1.4]);xlabel('Time[s]','Fontsize',16);
ylabel('Disturbance and Estimate','Fontsize',16); 
title('Figure 5.5(a)');
set(gca,'Fontname','Times New Roman','FontSize',14); % 文字の設定
set(gcf,'color','w');   % 枠の周囲を白く変更する
%
figure(2);clf(2); % 速度と速度推定値の描画
plot(t,vc,'k--',t,hat_vc,'k-','LineWidth',1.2);grid on;
legend('Velocity','Velocity Estimate','Location','Southeast');
axis([0,t_end,-1.5,2.0]);xlabel('Time[s]','Fontsize',16);
ylabel('Velocity and Estimate','Fontsize',16); 
title('Figure 5.5(b)');
set(gca,'Fontname','Times New Roman','FontSize',14); % 文字の設定
set(gcf,'color','w');   % 枠の周囲を白く変更する

%
delete('*.asv');delete('*.slxc');





