%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  プログラム 5.2: 最小次元ディジタルDOB
%           (速度観測型)の例
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;clc;
%% 物理パラメータ
m=1.0; % 質量[kg]

%% 拡大系状態空間モデル
A=0;B=1/m;C=1;D=0;
Abar=[A -B;0,0];Bbar=[B;0];Cbar=[C,0];Dbar=0;
[n,~]=size(Abar);[~,m]=size(Bbar);[p,~]=size(Cbar);% 次数,入力数,出力数
T=0.01;[Adbar,Bdbar,Cdbar,Ddbar]=c2dm(Abar,Bbar,Cbar,Dbar,T);% ディジタル化
% or Adbar=[1,-T/m;0,1],Bdbar=[T/m;0] % この例では左記としても等価

%% 外乱オブザーバ設計
alpha_d=0.8; % ディジタルDOBの極を指定
Ld=-(1-alpha_d)*m/T;
Ad_hat=Adbar(2,2)-Ld*Adbar(1,2);    % Ad_hat=1+T*L/m=alpha
Bd_hat=Ad_hat*Ld+Adbar(2,1)-Ld*Adbar(1,1);
% Bd_hat=T*Ld^2/m=(1-alpha_d)^2*m/T
Cd_hat=[0;1];Dd_hat=[1;Ld];         % Dd_hat=[1;-(1-alpha_d)m/T]
Jd_hat=-Ld*Bdbar(1)+Bdbar(2);       % J_hat=-Ld*T/m=1-alpha_d

%% 離散化と制御系設計
[Ad,Bd,Cd,Dd]=c2dm(A,B,C,D,T);
Q=1.5;R=2;Fd=dlqr(Ad,Bd,Q,R); % LQ制御法での設計

%% シミュレーション
f=0.5;d=1.0;t_end=5.0;vc0=1.5;
open_system('sim_Figure_5_11_and_5_12_min_digital_DOB');
set_param('sim_Figure_5_11_and_5_12_min_digital_DOB','WideLines','on'); 
set_param('sim_Figure_5_11_and_5_12_min_digital_DOB','ShowLineDimensions','on');
y=sim('sim_Figure_5_11_and_5_12_min_digital_DOB');

%% 描画
% Scope'vc&_vc_hat'のログに変数名vc_dataを指定すること。
% Scope'd&\hat_d'のログにd_dataを指定すること。
% Scopeのログの保存形式を「時間付き構造体」に設定する
t=y.vc_data.time;t2=y.d_data.time;
vc=y.vc_data.signals(1).values;
hat_vc=y.vc_data.signals(2).values;
d=y.d_data.signals(1).values;
hat_d=y.d_data.signals(2).values;
%
figure(1);clf(1);plot(t2,d,'k--',t2,hat_d,'k-','LineWidth',1.2);grid on;
legend('Disturbance','Disturbanse Estimate','Location','Northeast');
axis([0,t_end,-0.5,2.0]);xlabel('Time[s]','Fontsize',16);
ylabel('Disturbance and Estimate','Fontsize',16); 
set(gca,'Fontname','Times New Roman','FontSize',14); % 文字の設定
set(gcf,'color','w');   % 枠の周囲を白く変更する
%
figure(2);clf(2);plot(t,vc,'k--',t,hat_vc,'k-','LineWidth',1.2);grid on;
legend('Velocity','Velocity Estimate','Location','Northeast');
axis([0,t_end,-0.5,2.0]);xlabel('Time[s]','Fontsize',16);
ylabel('Velocity and Estimate','Fontsize',16); 
set(gca,'Fontname','Times New Roman','FontSize',14); % 文字の設定
set(gcf,'color','w');   % 枠の周囲を白く変更する
%
delete('*.slxc');delete('*.asv');





