%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  プログラム 5.3: 最小次元ディジタルDOB
%           (位置観測型)の例
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;clc;
%% 物理パラメータ
m=1;T=0.01;  % 質量[kg]と制御周期[s]

%% 拡大系状態空間モデル
A=[0 1;0 0];B=[0;1/m];C=[1 0];D=0;T=0.01;
[Ad,Bd,Cd,Dd]=c2dm(A,B,C,D,T);
A_=[A,-B;zeros(1,2),0];B_=[B;0];C_=[C,0];% 拡大系の作成
[Ad_,Bd_,Cd_,Dd_]=c2dm(A_,B_,C_,D,T);
% or Ad=[1,T,-1/2*T^2/m;0,1,-T/m;0,0,1];Bd=[1/2*T^2/m;T/m;0];Cd=[1, 0,0];
% Wd=[0,1,0;0,0,1];Td=[C_d;W_d]=eye(3);inv(Td)*Ad*Td=Ad
A11=Ad_(1,1);A12=Ad_(1,2:3);A21=Ad_(2:3,1);A22=Ad_(2:3,2:3);
B1=Bd_(1);B2=Bd_(2:3);

%% 外乱オブザーバ設計
pole=[0.75,0.80];Ltemp=place(A22',A12',pole);Ld=Ltemp';
Ad_hat=A22-Ld*A12;Bd_hat=Ad_hat*Ld+A21-Ld*A11;
Jd_hat=-Ld*B1+B2;Cd_hat=[zeros(1,2);eye(2)];Dd_hat=[1;Ld];

%% 制御系設計
Q=diag([1e2,1e1]);R=1;Fd=dlqr(Ad,Bd,Q,R);

%% シミュレーション
t_end=5;d=1.0;x0=[0.0;0];
open_system("sim_Figure_5_14_and_5_15_min_dig_DOB_pos");
set_param('sim_Figure_5_14_and_5_15_min_dig_DOB_pos','WideLines','on'); 
set_param('sim_Figure_5_14_and_5_15_min_dig_DOB_pos','ShowLineDimensions','on');
y=sim('sim_Figure_5_14_and_5_15_min_dig_DOB_pos');

%% 描画
% Scope'd&_hat_d'のログの変数名をd_dataと設定する
% Scope'vc&_vs_hat'のログの変数名をv_dataと設定する
% Scopeのログの保存形式を「時間付き構造体」に設定する
t1=y.d_data.time;t2=y.v_data.time;
vc=y.v_data.signals(1).values;
vc_hat=y.v_data.signals(2).values;
d=y.d_data.signals(1).values;
d_hat=y.d_data.signals(2).values;
%
figure(1);clf(1);plot(t1,d,'k--',t1,d_hat,'k-','LineWidth',1.2);grid on;
legend('Disturbance','Disturbanse Estimate','Location','Northeast');
axis([0,t_end,-0.2,1.75]);xlabel('Time[s]','Fontsize',16);
ylabel('Disturbance and Estimate','Fontsize',16); 
set(gca,'Fontname','Times New Roman','FontSize',14); % 文字の設定
set(gcf,'color','w');   % 枠の周囲を白く変更する
%
figure(2);clf(2);plot(t2,vc,'k--',t2,vc_hat,'k-','LineWidth',1.2);grid on;
legend('Velocity','Velocity Estimate','Location','Northeast');
axis([0,t_end,-0.075,0.125]);
xlabel('Time[s]','Fontsize',16);
ylabel('Velocity and Estimate','Fontsize',16); 
set(gca,'Fontname','Times New Roman','FontSize',14); % 文字の設定
set(gcf,'color','w');   % 枠の周囲を白く変更する
%
delete('*.slxc');delete('*.asv');