%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% プログラム 5.4: 高次ディジタル外乱オブザーバの設計例
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;clc;
%% 物理パラメータ
m=2.0;c=0.10;  % 質量[kg], 粘性摩擦係数[N/(m/s)]
A=[0 1; 0 -c/m];B=[0;1/m];C=[1 0];D=0;
x0=[0.1;0.0];  % 位置と速度の初期値 xc(0)=0.1m,vc(0)=0.0m/s
[n,m]=size(B); % 次数と入力数

%% 制御周期とディジタル状態空間モデルの作成
T=0.01;[Ad,Bd,Cd,Dd]=c2dm(A,B,C,D,T);

%% 制御系設計
pole=[0.98,0.99];Fd=place(Ad,Bd,pole);% 極配置法による設計

%% 外乱の設定と終了時間 
p=2;          % 外乱の次元。設計者が定める。 
tend=5;       % 終了時間
dis_step=1.0; % ステップ状外乱の大きさ

%% p次元外乱の設計と評価のための設定 
N=Bd;% マッチング条件が成り立つと仮定する
H=1;Gamma=zeros(p+1,p+1);for i=1:p H=[H,0];end;
Gamma(1:p,2:p+1)=eye(p);% Gammaを作成
Gamma_d=expm(Gamma*T) % 連続系のGammaをexpm(Gamma*T)を用いてディジタル化(p.128)
Abar=[Ad,-N*H;zeros(p+1,n),Gamma_d];
Bbar=[Bd;zeros(p+1,1)];Cbar=[Cd,zeros(1,p+1)];

%% p次元ディジタルDOBの設計  
pole_ob=[];base_of_pole=0.85;delta_pole=0.02;
for i=0:n+p pole_ob=[pole_ob,base_of_pole+delta_pole*i]; end;
Htemp=place(Abar',Cbar',pole_ob);Hob=Htemp';% 極配置法での設計
Hd1=Hob(1:n);Hd2=Hob(n+1:n+p+1);% 状態推定と外乱推定のためのゲイン行列   
x_ob0=x0; % x ob0は状態推定値x hatの初期値
% 未知としてx_ob0=[0;0]とした場合と比較してみることを推奨。

%% シミュレーション
open_system('sim_Figure_5_18_and_5_19_high_order_digital_dob');
set_param('sim_Figure_5_18_and_5_19_high_order_digital_dob','WideLines','on'); 
set_param('sim_Figure_5_18_and_5_19_high_order_digital_dob','ShowLineDimensions','on');
z=sim('sim_Figure_5_18_and_5_19_high_order_digital_dob');

%% 描画
% Scope'd&d_hat'のログの変数名をdoutに設定する
% Scopeのログの保存形式を「時間付き構造体」に設定する
time=z.dout.time;
d=z.dout.signals(1).values;
d_est=z.dout.signals(2).values;
%
figure(1);clf(1);
plot(time,d,'k--',time,d_est,'k-','LineWidth',1.2);grid on;
legend('Disturbance','Disturbanse Estimate','Location','Southeast');
axis([0,tend,-0.2,6]);xlabel('Time[s]','Fontsize',16);
ylabel('Disturbance and Estimate','Fontsize',16); 
title('Figure 5.19 (a) or (b)')
set(gca,'Fontname','Times New Roman','FontSize',14); % 文字の設定
set(gcf,'color','w');   % 枠の周囲を白く変更する

delete('*.slxc');delete('*.asv');
