%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% プログラム 6.1: 単純なDOBを用いた2慣性系の外乱推定と制御
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;clc;
%% 物理パラメータ
% 回転角度:th1,th2,回転速度:w1,w2
J1=1.0;J2=2-J1;    % 入力軸と出力軸の慣性モーメント 
k=1e3;c=0.1;       % 弾性係数と粘性摩擦係数
tau_d2=10;T_cyc=2; % 出力軸への矩形波状外乱の大きさと周期

%% DOBとPI制御器の設計
T=0.01;a=1/T;          % DOBの時定数
zeta=7.5;wn=2.5;       % 制御系のための減衰定数と応答角周波数
Kp=2*zeta*wn;Ki=wn^2;  % PI制御ゲイン

%% シミュレーション
th10=0;th20=th10; % 角度の初期値
tend=5;           % 終了時間
open_system('sim_Figure_6_4_and_6_5_two_mass_control_with_simple_DOB'); 
y=sim('sim_Figure_6_4_and_6_5_two_mass_control_with_simple_DOB');

%% 描画
% Scope'd&d_hat'のログの変数名をd_outに設定する
% Scope'w1&w2'のログの変数名をw_outに設定する
% Scopeのログの保存形式を「時間付き構造体」に設定する
time=y.d_out.time;
vc_ref=y.w_out.signals(1).values;
vc1=y.w_out.signals(2).values;
vc2=y.w_out.signals(3).values;
d_data=y.d_out.signals(1).values;
d_hat=y.d_out.signals(2).values;
% 
figure(1);clf(1);
plot(time,d_data,'--k',time,d_hat,'k','LineWidth',1.2);grid on;
xlabel('Time [s]','Fontsize',16) 
ylabel('Disturbance & Estimate[Nm]','Fontsize',16);
axis([0,tend,-25,25]);
legend(' \tau_{d2}(Real)',' \tau_{d1}(Estimate)','Location','Southeast');
title(['Disturbance d2 and Esimate d1 for Figure 6.4']);
set(gca,'Fontname','Times New Roman','FontSize',14); % 文字の設定
set(gcf,'color','w');   % 枠の周囲を白く変更する
%
figure(2);clf(2);
plot(time,vc_ref,'--k',time,vc1,'k-.',time,vc2,'k','LineWidth',1.2);grid on;
xlabel('Time [s]','Fontsize',16)  
ylabel('Angular Velocity[rad/s]','Fontsize',16);
axis([0,tend,-0.2,1.8]);  
legend('Velocity-ref',' Velocity-1',' Velocity-2','Location','Southeast');
title('Figure 6.5 (a) or (b)');
set(gca,'Fontname','Times New Roman','FontSize',14); % 文字の設定g
set(gcf,'color','w');   % 枠の周囲を白く変更する

delete('*.slxc');delete('*.asv');
