%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% プログラム 6.1a: 単純なDOBを用いた2慣性系の
%      DOB併用速度制御系のボード線図
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%　プログラムリストは本では省略。図6.6に結果の一部のみ掲載。
%------------------------------------------------
clear all;clc;close all;
%%　物理パラメータ
% 回転角度:th1,th2,回転速度:w1,w2
J1=1.0;J2=2-J1; % 入力軸と出力軸の慣性モーメント
k=1e3;c=0.1;    % 弾性係数と粘性摩擦係数

%% 制御系設計
zeta=7.5;wn=2.5;       % 制御系のための減衰定数と応答角周波数
Kp=2*zeta*wn;Ki=wn^2;  % PI制御ゲイン
%% DOBの時定数を5種類定義し, Simulinkモデルを開く
Time_constant=[0.001,0.005,0.05,0.5,5]; % DOBの時定数を5種類定義しておく
open_system('sim_Figure_6_6_bode_of_open_loop_two_mass_with_simple_DOB');

%% linmod関数を用いたボード線図の例（linmodはLinear modelを作成する関数)
for i=1:5
  T=Time_constant(i);a=1/T; % i番目のDOBの時定数をTとおく
  [A,B,C,D]=linmod('sim_Figure_6_6_bode_of_open_loop_two_mass_with_simple_DOB');
  [num,den]=ss2tf(A,B,C,D);
  min=1;max=2;w=logspace(min,max,300); % 角周波数の範囲
  % 10^{-1}radから10^{2}rad/sまで
  [gain,phase]=bode(num,den,w);

  %% PI制御系のw1/角度偏差eの開ループボード線図の描画
  figure(1);
  subplot(2,1,1);semilogx(w,20*log10(gain(:,1)),'k-','LineWidth',1.0);
  grid on;hold on;  axis([10^(min),10^(max),-40,40]);ylabel('gain[dB]')
    subplot(2,1,2);semilogx(w,phase(:,1),'k-','LineWidth',1.0);hold on;
  grid on;hold on;  axis([10^(min),10^(max),-270,90]);
  ylabel('Phase[deg]');xlabel('Frequency[rad/s]')

 %%  PI制御系のw2/角度偏差eの開ループボード線図の描画
 figure(2);
 subplot(2,1,1);semilogx(w,20*log10(gain(:,2)),'k-','LineWidth',1.0);hold on;
 grid on;hold on; axis([10^(min),10^(max),-40,40]);  ylabel('gain[dB]')
 subplot(2,1,2);semilogx(w,phase(:,2),'k-','LineWidth',1.0);hold on;
 grid on;hold on; axis([10^(min),10^(max),-360,0]);
 ylabel('Phase[deg]');xlabel('Frequency[rad/s]')
end;

figure(1);subplot(2,1,1);
title('Bode diagram of w1/e of 2 Inertia System');
set(gca,'Fontname','Times New Roman','FontSize',14); % 文字の設定
subplot(2,1,2);
set(gca,'Fontname','Times New Roman','FontSize',14); % 文字の設定
set(gcf,'color','w');   % 枠の周囲を白く変更する
figure(2);subplot(2,1,1);
title('Figure 6.6: Bode diagram of w2/e');
set(gca,'Fontname','Times New Roman','FontSize',14); % 文字の設定
subplot(2,1,2);
set(gca,'Fontname','Times New Roman','FontSize',14); % 文字の設定
set(gcf,'color','w');   % 枠の周囲を白く変更する

delete('*.slxc');delete('*.asv');delete('*.autosave');