%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  プログラム 6.2: 2慣性系の入力軸外乱の推定と外乱抑制
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;clc;
%%　物理パラメータ
% 回転角度:th1,th2,回転速度:w1,w2
J1=1.0;J2=2-J1;    % 入力軸と出力軸の慣性モーメント 
k=1;c=0.01;       % 弾性係数と粘性摩擦係数
tau_d1=10;T_cyc=5; % 入力軸への矩形波状外乱の大きさと周期

%% 状態空間モデルの作成
A=[0,     0,  1,0; 
   0,     0,  0,1;
   -k/J1, k/J1,-c/J1, c/J1; 
    k/J2,-k/J2, c/J2,-c/J2];
B1=[0;0;1/J1;0];
C1=[1,0,0,0]; % C1行列は入力軸の観測に対応

%% 拡大系の作成とDOB設計
A_bar=[A,-B1;zeros(1,5)];C1_bar=[C1,0];
% pole_dob=[-25,-27,-30,-35,-50]; DOBの極
% H_temp=place(A_bar',C1_bar',pole_dob); % 極配置法によるDOB設計
Q_ob=diag([1e2,1e2,1e1,1e1,1e8]);R_ob=1;
H_temp=lqr(A_bar',C1_bar',Q_ob,R_ob);    % LQ法によるDOB設計
H1=H_temp(1:4)';H2=H_temp(5); % 第1～4要素は状態変数xの推定用ゲイン, 
% 第5要素は外乱推定用ゲイン

%% LQ制御系設計
Q=diag([1e1,1e3,1e2,1e2]);R=1;F=lqr(A,B1,Q,R);

%% シミュレーション
th10=pi/3;th20=th10;x0=[th10;th20;0;0]; % Initial values
tend=10;
open_system('sim_Figure_6_8_and_6_9_DOB_for_two_inertia_system_with_taud1');
set_param('sim_Figure_6_8_and_6_9_DOB_for_two_inertia_system_with_taud1','WideLines','on'); 
set_param('sim_Figure_6_8_and_6_9_DOB_for_two_inertia_system_with_taud1','ShowLineDimensions','on');
y=sim('sim_Figure_6_8_and_6_9_DOB_for_two_inertia_system_with_taud1');

%% 描画
% Scope'd&d_hat'のログの変数名をd_outに設定する
% Scope'th1&th2'のログの変数名をy_outに設定する
% Scopeのログの保存形式を「時間付き構造体」に設定する
time=y.d_out.time;
d_data=y.d_out.signals(1).values;d_hat=y.d_out.signals(2).values;
% 外乱と外乱推定値の描画
figure(1);clf(1);
plot(time,d_data,'--k',time,d_hat,'k','LineWidth',1.2);grid on;
xlabel('Time [s]','Fontsize',16); 
ylabel('Disturbance & Estimate[Nm]','Fontsize',16);
axis([0,tend,-1,12]);
legend(' \tau_{d1}(Real)',' \tau_{d1}(Estimate)','Location','Northwest') %凡例
title('Figure 6.9 (a)');
set(gca,'Fontname','Times New Roman','FontSize',14); % 文字の設定
set(gcf,'color','w');   % 枠の周囲を白く変更する
% 
th1=y.y_out.signals(1).values;th2=y.y_out.signals(2).values;
figure(2);clf(2);
plot(time,th1,'--k',time,th2,'k','LineWidth',1.2);grid on;
xlabel('Time [s]','Fontsize',16); 
ylabel('Angle[rad]','Fontsize',16);
axis([0,tend,-0.5,1.2]); 
legend(' \theta_1',' \theta_2','Location','Northeast');
title('Figure 6.9 (b)');
set(gca,'Fontname','Times New Roman','FontSize',14); % 文字の設定
set(gcf,'color','w');   % 枠の周囲を白く変更する

delete('*.slxc');delete('*.asv');




