%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  プログラム 6.5: 2慣性系の出力軸外乱の推定と出力軸サーボ系
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;clc;format short e;
%%　物理パラメータ
% 回転角度:th1,th2,回転速度:w1,w2
J1=1.0;J2=2-J1;       % 入力軸と出力軸の慣性モーメント　
k=1;c=0.0001;         % 弾性係数と粘性摩擦係数 
tau_d2=0.25;T_cyc=18; % 出力軸への矩形波状外乱の大きさと周期

%% 拡大系の作成
A=[0,     0,  1,0; 
   0,     0,  0,1;
   -k/J1, k/J1,-c/J1, c/J1; 
    k/J2,-k/J2, c/J2,-c/J2];
B1=[0;0;1/J1;0];B2=[0;0;0;1/J2]; % B2行列は出力軸への外乱に対応
C1=[1,0,0,0];                    % C1行列は入力軸の観測に対応
C2=[0,1,0,0];                    % C2行列は出力軸の観測に対応

%% Create extended system and design DOB
A_bar=[A,-B2;zeros(1,5)];C1_bar=[C1,0];
pole_dob=[-30,-32,-34,-36,-39]; 
H_temp=place(A_bar',C1_bar',pole_dob); % 極配置でのDOB設計
H1=H_temp(1:4)';H2=H_temp(5); % 第1～4要素は状態変数xの推定用ゲイン, 
% 第5要素は外乱推定用ゲイン

%% 出力軸サーボ系設計
A_tilde=[A,zeros(4,1);-C2,0];B_tilde=[B1;0];
Q=diag([1e2,1e2,1e4,1e5,1e5]);R=1;
F_tilde=lqr(A_tilde,B_tilde,Q,R);F=F_tilde(1:4);Ki=-F_tilde(5);

%% シミュレーション
th10=0;th20=th10;x0=[th10;th20;0;0]; % 初期値
tend=20;
open_system('sim_Figure_6_14_and_6_15_two_inertia_servo_taud2');
set_param('sim_Figure_6_14_and_6_15_two_inertia_servo_taud2','WideLines','on'); 
set_param('sim_Figure_6_14_and_6_15_two_inertia_servo_taud2','ShowLineDimensions','on');
y=sim('sim_Figure_6_14_and_6_15_two_inertia_servo_taud2');

%% 描画
% Scope'd&d_hat'のログの変数名をd_outに設定する
% Scope'th1&th2'のログの変数名をy_outに設定する
% Scopeのログの保存形式を「時間付き構造体」に設定する
time=y.d_out.time;
d_data=y.d_out.signals(1).values;d_hat=y.d_out.signals(2).values;
% 外乱と外乱推定値の描画
figure(1);clf(1);
plot(time,d_data,'--k',time,d_hat,'k','LineWidth',1.2);grid on;
xlabel('Time [s]','Fontsize',16)  
ylabel('Disturbance & Estimate[Nm]','Fontsize',16);
axis([0,tend,-0.05,0.3]);
legend(' \tau_{d2}(Real)',' \tau_{d2}(Estimate)','Location','Northwest');
title('Figure 6.15 (a)');
set(gca,'Fontname','Times New Roman','FontSize',14); % 文字の設定
set(gcf,'color','w');   % 枠の周囲を白く変更する

% 出力の描画
th1=y.y_out.signals(1).values;th2=y.y_out.signals(2).values;
th1_hat=y.hat_data.signals(1).values;th2_hat=y.hat_data.signals(2).values;
figure(2);clf(2);
plot(time,th1,'--k',time,th2,'k',time,th1_hat,'-.k',time,th2_hat,'k:','LineWidth',1.2);grid on;
xlabel('Time [s]','Fontsize',16);ylabel('Angle[rad]','Fontsize',16);
axis([0,tend,-0.25,1.5]);
legend(' \theta_1',' \theta_2','\theta_1-est','\theta_2-est','Location','Southeast');
title('Figure 6.15 (b)');
set(gca,'Fontname','Times New Roman','FontSize',14); % 文字の設定
set(gcf,'color','w');   % 枠の周囲を白く変更する

delete('*.slxc');delete('*.asv');

