%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% プログラム 7.1 スミス法と通信外乱オブザーバ併用制御系の例
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 外乱d(t)のManual Switchの切り替えで外乱の影響を確認すること
clear all;clc;
%% 物理パラメータとPID制御系設計
m=2;c=1;            % 質量[kg]と粘性摩擦係数[N/(m/s)]
L=0.2;L_est=L*0.25; % 通信時間遅れ:L, 通信時間遅れ推定値:L_est　
w=2;                % 位置参照値の角周波数[rad/s]
Kp=30;Ki=5;Kd=15;   % PIDゲイン

%% 状態空間表現でのCDOB設計
A=[0 1;0 -c/m];B=[0; 1/m];C=[1,0];
Abar=[A,-B;[0,0],0];Cbar=[C,0];
pole=[-10,-12, -13]*10;
Htemp=place(Abar',Cbar',pole);H=Htemp';
H1=H(1:2);H2=H(3);

%% シミュレーション 
tend=20;t_dis=tend*2/3;  % 終了時間と外乱発生時間
dis=0.5;                 % 外乱の大きさ
open_system('sim_Figure_7_3_7_4_and_7_5_Smith_method_and_CDOB');
z=sim('sim_Figure_7_3_7_4_and_7_5_Smith_method_and_CDOB');

%% 描画
% Scope'Pos_Smith'のログの変数名をpos_outに設定する
% Scope'L_out'のログの変数名をL_outに設定する
% Scope'Pos_CDOB'のログの変数名をpos_CDOB_outに設定する
% Scope'cd_hat1'のログの変数名をL_hat_outに設定する
% Scopeのログの保存形式を「時間付き構造体」に設定する
t=z.pos_out.time;
xc_ref=z.pos_out.signals(1).values;
xc=z.pos_out.signals(2).values;
du=z.L_out.signals(1).values;
du_est=z.L_out.signals(2).values;
%
xc_CDOB_ref=z.pos_CDOB_out.signals(1).values;
xc_CDOB=z.pos_CDOB_out.signals(2).values;
du_=z.L_hat_out.signals(1).values;
du_hat=z.L_hat_out.signals(2).values;
%
figure(1);clf(1);plot(t,xc_ref,'k--',t,xc,'k-','LineWidth',1.2);grid on;
xlabel('Time[s]','Fontsize',16);ylabel('Position[m]','Fontsize',12);
axis([0,tend,-2,2]);
legend({'Position reference','Position'},'Location','Northeast','Fontsize',12);
title('Figure 7.5 (a-1)');
set(gca,'Fontname','Times New Roman','FontSize',14); % 文字の設定
set(gcf,'color','w');   % 枠の周囲を白く変更する
%
figure(2);clf(2);plot(t,du,'k--',t,du_est,'k-','LineWidth',1.2);grid on;
xlabel('Time[s]','Fontsize',16);ylabel('u(t)-u(t-L)[N]','Fontsize',12);
axis([0,tend,-10,10]);
legend({'u(t)-u(t-L)','u(t)-u(t-Lest)'},'Location','Northeast','Fontsize',12);
title('Figure 7.5 (b-1)');
set(gca,'Fontname','Times New Roman','FontSize',14); % 文字の設定
set(gcf,'color','w');   % 枠の周囲を白く変更する
%
figure(3);clf(3);plot(t,xc_CDOB_ref,'k--',t,xc_CDOB,'k-','LineWidth',1.2);grid on;
xlabel('Time[s]','Fontsize',16);ylabel('Position[m]','Fontsize',12);
axis([0,tend,-2,2]);
legend({'Positio reference', 'Position'},'Location','Northeast','Fontsize',12);
title('Figure 7.5 (a-2)');
set(gca,'Fontname','Times New Roman','FontSize',14); % 文字の設定
set(gcf,'color','w');   % 枠の周囲を白く変更する
%
figure(4);clf(4);plot(t,du_,'k--',t,du_hat,'k-','LineWidth',1.2);grid on;
xlabel('Time[s]','Fontsize',16);ylabel('u(t)-u(t-L)[N]','Fontsize',12);
axis([0,tend,-10,10]);
legend({'u(t)-u(t-L)','u(t)-u(t-L_hat)'},'Location','Northeast','Fontsize',12);
title('Figure 7.5 (b-2)');
set(gca,'Fontname','Times New Roman','FontSize',14); % 文字の設定
set(gcf,'color','w');   % 枠の周囲を白く変更する

delete('*.slxc');delete('*.asv');delete('*.autosave');
